#include <PalmOS.h>
#include <Hs.h>
#include <PalmOnePhoto.h>
#include <HsPhonePower.h>
#include <HsPhoneNetwork.h>
#include <HsPhoneLibrary.h>
#include <PmSysGadgetLib.h>
#include <HsKeyCodes.h>
#include <BtLib.h>

#include "NeoLauncher.h"
#include "NeoLauncher_Rsc.h"
#include "Setup.h"

static void drawPhoneUIBackground(SettingType *settingP);
static void drawPhoneUIStatusBar(SettingType *settingP);
static Boolean frmPhoneUICustomEventHandler(EventType *eventP, SettingType *settingP);

static Boolean isRunningOnTreo(SettingType *settingP);
static void notifyUnregister(SettingType *settingP);
static void notifyRegister(SettingType *settingP);

static void launchProgram(ProgramListType *programInfo, UInt16 selectedProgram, SettingType *settingP);
static void freePluginList(PluginListType *pluginListP, UInt16 pluginListCount);
static void getPluginList(PluginListType **pluginListPP, UInt16 *pluginListCount, Boolean isActivedOnly);
static void paintCharWithOutline(Char chr, Coord x, Coord y, RGBColorType *outlineColorP);
static void paintCharsWithOutline(const Char *stringP, UInt16 stringLength, Coord x, Coord y, RGBColorType *outlineColorP);
static void winPaintTruncChars(Char *stringP, UInt16 stringLength, Coord x, Coord y, UInt16 maxWidth, Boolean shouldDrawOutline, RGBColorType *outlineColorP);
static void convertCharsToBitPackage(Char *srcStringP, UInt32 *pyTableP, UInt32 *destUInt32BitPackageP, UInt16 *UInt32BitPackageLengthP);
static Char *strPreventChr(Char *stringP, Char chr);
static void freeGlobeCategoryList();
static void freeGlobeProgramList();
static void freeGlobeProgramList();
static WChar keyTransfer(WChar chr, UInt16 keyCode, UInt16 modifiers);
static WChar customKey();
static void getDefaultColor(SettingType *settingP);

#if 0
#pragma mark -
#pragma mark GLOBE VARIBLES
#pragma mark -
#endif

GlobeType	*globeP;

#if 0
#pragma mark -
#pragma mark PROGRAM LIST FUNCTIONS
#pragma mark -
#endif

//ZLĻص
static void NeoZLReopen(ZLPluginLaunchCmdBlockType * blockP)
{
	return;
}

static void NeoZLBeginDraw(ZLPluginLaunchCmdBlockType * blockP, Boolean	lockScreen)
{
	return;
}

static void NeoZLEndDraw(ZLPluginLaunchCmdBlockType * blockP)
{
	return;
}

static void NeoZLPrepareHandlingEvent(ZLPluginLaunchCmdBlockType * blockP)
{
	return;
}

static void NeoZLSetHint(ZLPluginLaunchCmdBlockType * blockP, Char * hints)
{
	return;
}

static void	NeoZLShowHint(ZLPluginLaunchCmdBlockType * blockP, Coord x, Coord y)
{
	return;
}

static void	NeoZLHideHint(ZLPluginLaunchCmdBlockType * blockP)
{
	return;
}

static UInt32 NeoZLPrefsReadValue(ZLPluginLaunchCmdBlockType * blockP, UInt16 id, UInt32 defaultValue)
{
	ZLPluginLaunchCmdBlockTypeEx *blockExP = (ZLPluginLaunchCmdBlockTypeEx *)blockP;
	UInt32 value;
	UInt16 size = sizeof(UInt32);

	if (PrefGetAppPreferences(blockExP->creatorId, id, &value, &size, true) != noPreferenceFound)
		return value;

	return defaultValue;
}

static UInt32 NeoZLPrefsReadVariableValue(ZLPluginLaunchCmdBlockType * blockP, UInt16 id, void * valueP, UInt16 * sizeP)
{
	ZLPluginLaunchCmdBlockTypeEx *blockExP = (ZLPluginLaunchCmdBlockTypeEx *)blockP;

	if (PrefGetAppPreferences(blockExP->creatorId, id, valueP, sizeP, true) != noPreferenceFound)
		return *sizeP;

	return noPreferenceFound;
}

static UInt16 NeoZLPrefsWriteValue(ZLPluginLaunchCmdBlockType * blockP, UInt16 id, UInt32 value)
{
	ZLPluginLaunchCmdBlockTypeEx *blockExP = (ZLPluginLaunchCmdBlockTypeEx *)blockP;
	UInt16 size = sizeof(value);

	PrefSetAppPreferences(blockExP->creatorId, id, 1, &value, size, true);

	return size;
}

static UInt16 NeoZLPrefsWriteVariableValue(ZLPluginLaunchCmdBlockType * blockP, UInt16 id, void * valueP, UInt16 size)
{
	ZLPluginLaunchCmdBlockTypeEx *blockExP = (ZLPluginLaunchCmdBlockTypeEx *)blockP;

	PrefSetAppPreferences(blockExP->creatorId, id, 1, valueP, size, true);

	return size;
}

static void	NeoZLRefreshBackground(ZLPluginLaunchCmdBlockType * blockP)
{
	return;
}

static void	NeoZLDrawChars(ZLPluginLaunchCmdBlockType * blockP, Char * chars, Coord x, Coord y, Coord width)
{
	WinPaintChars(chars, StrLen(chars), x, y);
}

static void	NeoZLDrawBitmap(ZLPluginLaunchCmdBlockType * blockP, BitmapType * pBitmap, Coord x, Coord y)
{
	WinPaintBitmap(pBitmap, x, y);
}

//Ƿд濨
static Boolean isCardInserted()
{
	UInt16			volRef			= 0;
	UInt32			volIterator		= vfsIteratorStart;

	while (volIterator != vfsIteratorStop)
	{
		VFSVolumeEnumerate(&volRef, &volIterator);
	}

	if (volRef > 0)
	{
		return true;
	}

	return false;
}

//ȡȫ
static void buildCategoryList()
{
	UInt16			i, recordNums;
	MemHandle		recordH;
	CategoryType	*recordP;
	DmOpenRef		dbRef		= DmOpenDatabaseByTypeCreator(CATEGORY_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadOnly);

	//ͷз
	freeGlobeCategoryList();

	if (dbRef != NULL)
	{
		//
		recordNums = DmNumRecordsInCategory(dbRef, dmAllCategories);
		i = recordNums + 2;
		globeP->categoryListPP = (Char **)MemPtrNew(i * 4);
		globeP->categoryListMappingP = (UInt8 *)MemPtrNew(i);
		MemSet(globeP->categoryListMappingP, i, 0xFF);
		globeP->hideCategoryP = (UInt8 *)MemPtrNew(i);
		MemSet(globeP->hideCategoryP, i, 0xFF);

		//ȫ
		globeP->categoryListPP[0] = (Char *)MemPtrNew(StrLen(STRING_ALL_CATEGORIER) + 1);
		StrCopy(globeP->categoryListPP[0], STRING_ALL_CATEGORIER);
		globeP->categoryListMappingP[0] = ALL_CATEGORIES;
		globeP->categoryCount = 1;
		globeP->hideCategoryCount = 0;

		for(i = 0; i < recordNums; i ++)
		{
			recordH = DmQueryRecord(dbRef, i);
			if (recordH == NULL)
				break;
			recordP = (CategoryType *)MemHandleLock(recordH);
			if (recordP == NULL)
			{
				MemHandleUnlock(recordH);
				break;
			}			
			if (! recordP->isCategoryHide)
			{
				globeP->categoryListPP[globeP->categoryCount] = (Char *)MemPtrNew(StrLen(recordP->nameP) + 1);
				StrCopy(globeP->categoryListPP[globeP->categoryCount], recordP->nameP);
				globeP->categoryListMappingP[globeP->categoryCount] = recordP->categoryId;
				globeP->categoryCount ++;
			}
			else
			{
				globeP->hideCategoryP[globeP->hideCategoryCount] = recordP->categoryId;
				globeP->hideCategoryCount ++;
			}
			MemHandleUnlock(recordH);
		}

#if 0
		//濨
		if (isCardInserted())
		{
			globeP->categoryListPP[globeP->categoryCount] = (Char *)MemPtrNew(StrLen(STRING_CARD) + 1);
			StrCopy(globeP->categoryListPP[globeP->categoryCount], STRING_CARD);
			globeP->categoryListMappingP[globeP->categoryCount] = CARD_CATEGORY;
			globeP->categoryCount ++;
		}
#endif
		DmCloseDatabase(dbRef);
	}
}

//ȡķ
static void getProgramCategory(ProgramListType *dbInfoP)
{
	Char				*rscP;
	MemHandle			rscH;
	CategoryType		*recordP;
	MemHandle			recordH;
	UInt16				recordId;
	UInt16				i;
	UInt16				recordNums;
	UInt8				maxCategoryId;
	DmOpenRef			cDbRef;
	DmOpenRef			dbRef	= DmOpenDatabaseByTypeCreator(dbInfoP->type, dbInfoP->creatorId, dmModeReadOnly);

	//ȡԴ
	rscH = DmGet1Resource('taic', 1000);
	if (rscH == NULL)
	{
		dbInfoP->categoryId = 0;
	}
	else
	{
		rscP = (Char *)MemHandleLock(rscH);

		if (rscP != NULL)
		{
			//򿪷
			cDbRef = DmOpenDatabaseByTypeCreator(CATEGORY_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadWrite);
			if (DmGetLastErr() != errNone)	//ڷ
			{
				DmCreateDatabase(0, "Category List", APPLICATION_CREATOR, CATEGORY_LIST_DB_TYPE, false);
				cDbRef = DmOpenDatabaseByTypeCreator(CATEGORY_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadWrite);
				if (cDbRef != NULL)
				{
					recordId = dmMaxRecordIndex;
					recordH = DmNewRecord(cDbRef, &recordId, sizeof(CategoryType));
					if (recordH != NULL)
					{
						recordP = (CategoryType *)MemHandleLock(recordH);
						if (recordP != NULL)
						{
							DmSet(recordP, 0, sizeof(CategoryType), 0);
							DmStrCopy(recordP, 2, STRING_UNFILED_CATEGORY);
						}
						MemHandleUnlock(recordH);
						DmReleaseRecord(cDbRef, recordId, true);
					}
				}
			}

			//÷ڷı
			recordNums = DmNumRecordsInCategory(cDbRef, dmAllCategories);
			maxCategoryId = 0;
			dbInfoP->categoryId = 0;
			for (i = 0; i < recordNums; i ++)
			{
				recordH = DmQueryRecord(cDbRef, i);
				if (recordH == NULL)
					break;
					
				recordP = (CategoryType *)MemHandleLock(recordH);
				if (recordP == NULL)
				{
					MemHandleUnlock(recordH);
					break;
				}
				if (maxCategoryId < recordP->categoryId)
				{
					maxCategoryId = recordP->categoryId;
				}
				if (StrCompare(rscP, recordP->nameP) == 0)	//ҵ
				{
					dbInfoP->categoryId = recordP->categoryId;
					MemHandleUnlock(recordH);
					break;
				}
				MemHandleUnlock(recordH);
			}
			//ݳϢµķ
	#if 0		
			if (dbInfoP->categoryId == 0)	//ûжӦķ࣬½
			{
				maxCategoryId ++;
				recordId = dmMaxRecordIndex;
				recordH = DmNewRecord(cDbRef, &recordId, sizeof(CategoryType));
				recordP = (CategoryType *)MemHandleLock(recordH);
				DmSet(recordP, 0, sizeof(CategoryType), 0);
				DmWrite(recordP, 1, &maxCategoryId, 1);
				DmWrite(recordP, 2, rscP, StrLen(rscP));
				MemHandleUnlock(recordH);
				DmReleaseRecord(cDbRef, recordId, true);
				dbInfoP->categoryId = maxCategoryId;
			}
	#endif
			//رշ
			if (cDbRef != NULL)
				DmCloseDatabase(cDbRef);

			MemHandleUnlock(rscH);
		}

		DmReleaseResource(rscH);
	}

	if (dbRef != NULL)
		DmCloseDatabase(dbRef);
}

//ȡصķ
static Boolean isCategoryHide(UInt8 categoryId)
{
	UInt8		i;

	for (i = 0; i < globeP->hideCategoryCount; i ++)
	{
		if (globeP->hideCategoryP[i] == categoryId)
		{
			return true;
		}
	}

	return false;
}

//ȡбӦķ
static UInt8 getCategoryId(Int16 listIndex, UInt16 listCount)
{
	return globeP->categoryListMappingP[listIndex];
}

#if 0
#pragma mark -----------------------------------------------------
#endif

//򿪳бݿ
static DmOpenRef openProgramDatabase()
{
	DmOpenRef		dbRef	= DmOpenDatabaseByTypeCreator(PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadWrite);

	if (DmGetLastErr() != errNone)
	{
		DmCreateDatabase(0, "Program List", APPLICATION_CREATOR, PROGRAM_LIST_DB_TYPE, false);
		dbRef = DmOpenDatabaseByTypeCreator(PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadWrite);
	}

	return dbRef;
}

//ȡָĳڳбݿеλ
static Err getProgramId(DmOpenRef dbRef, UInt32 creatorId, UInt16 *recordId)
{
	MemHandle			recordH;
	ProgramListType		*recordP;
	UInt16				recordNum	= DmNumRecordsInCategory(dbRef, dmAllCategories);

	for ((*recordId) = 0; (*recordId) < recordNum; (*recordId) ++)
	{
		recordH = DmQueryRecord(dbRef, (*recordId));
		recordP = (ProgramListType *)MemHandleLock(recordH);
		if (recordP->creatorId == creatorId)
		{
			MemHandleUnlock(recordH);
			return errNone;
		}
		MemHandleUnlock(recordH);
	}

	return ERROR_PROGRAM_NOT_FOUND;
}

//ĳǷڳб
static Boolean isDbInDbList(DmOpenRef dbRef, UInt32 creatorId, Char *VFSNameP, UInt16 *recordIdP)
{
	UInt16				i;
	MemHandle			recordH;
	ProgramListType		*recordP;
	UInt16				recordNum	= DmNumRecordsInCategory(dbRef, dmAllCategories);

	for (i = 0; i < recordNum; i ++)
	{
		recordH = DmQueryRecord(dbRef, i);
		if (recordH != NULL)
		{
			recordP = (ProgramListType *)MemHandleLock(recordH);
			if (recordP != NULL)
			{
				//ڴͿͬ
				if
				(
					(recordP->creatorId == creatorId)
					&&
					(
						(
							recordP->VFSNameP[0] == '\0'
							&&
							VFSNameP == NULL
						)
						||
						(
							VFSNameP != NULL
							&&			
							(StrCompare(recordP->VFSNameP, VFSNameP) == 0)
						)
					)
				)
				{
					MemHandleUnlock(recordH);
					if (recordIdP != NULL)
					{
						(*recordIdP) = i;
					}
					return true;
				}
				MemHandleUnlock(recordH);
			}	
		}	
	}

	return false;
}

//򻺴
static void buildProgramList(UInt8 mode, Boolean updateCardPrograms)
{
	UInt16				i;
	MemHandle			dbListH;
	SysDBListItemType	*dbListP;
	UInt16				dbListCount;
	ProgramListType		dbInfo;
	ProgramListType		*recordP;
	UInt16				attributes;
	MemHandle			recordH;
	UInt16				recordId;
	DmSearchStateType	dmSearchState;
	DmOpenRef			dbRef			= NULL;
	Char				nameP[128];

	switch (mode)
	{
		case BUILD_LIST_MODE_INIT:
			//ɾеķͳб
			if (DmGetNextDatabaseByTypeCreator(true, &dmSearchState, CATEGORY_LIST_DB_TYPE, APPLICATION_CREATOR, true, &dbInfo.cardNo, &dbInfo.dbId) != dmErrCantFind)
			{
				DmDeleteDatabase(dbInfo.cardNo, dbInfo.dbId);
			}
			if (DmGetNextDatabaseByTypeCreator(true, &dmSearchState, PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, true, &dbInfo.cardNo, &dbInfo.dbId) != dmErrCantFind)
			{
				DmDeleteDatabase(dbInfo.cardNo, dbInfo.dbId);
			}

			//򿪳б
			dbRef = openProgramDatabase();

			if (dbRef != NULL)
			{
				//ȡڴеĳб
				if (SysCreateDataBaseList('appl', 0, &dbListCount, &dbListH, true))
				{
					dbListP = (SysDBListItemType *)MemHandleLock(dbListH);
					if (dbListP != NULL)
					{
						for (i = 0; i < dbListCount; i ++)
						{
							if (dbListP[i].creator != APPLICATION_CREATOR && dbListP[i].creator != sysFileCLauncher/* && dbListP[i].creator != PREF_CREATOR*/)
							{
								if (DmDatabaseInfo(dbListP[i].cardNo, dbListP[i].dbID, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == errNone)
								{
									if ((! (attributes & dmHdrAttrHidden)))
									{
										dbInfo.cardNo = dbListP[i].cardNo;
										dbInfo.dbId = dbListP[i].dbID;
										dbInfo.type = dbListP[i].type;
										dbInfo.creatorId = dbListP[i].creator;
										
										//
										StrCopy(dbInfo.programNameP, dbListP[i].name);
										
										//ȡϢ
										getProgramCategory(&dbInfo);

										dbInfo.shortcut = chrNull;
										dbInfo.addOnInfo = 0;
										dbInfo.VFSNameP[0] = '\0';

										//
										recordId = dmMaxRecordIndex;
										recordH = DmNewRecord(dbRef, &recordId, sizeof(ProgramListType));
										if (recordH != NULL)
										{
											recordP = (ProgramListType *)MemHandleLock(recordH);
											if (recordP != NULL)
												DmWrite(recordP, 0, &dbInfo, sizeof(ProgramListType));
											MemHandleUnlock(recordH);
											DmReleaseRecord(dbRef, recordId, true);
										}	
									}
								}
							}
						} // for loop
					} // dbListP != NULL
				}// dbRef != NULL
				
				//ȡϵĳ
				{
					FileInfoType	fileInfo;
					FileRef			fileRef;
					UInt32			type;
					UInt32			creatorId;
					UInt16			attributes;
					UInt16			volRef			= 0;
					UInt32			dirRef			= 0;
					UInt32			volIterator		= vfsIteratorStart;
					UInt32			dirIterator		= vfsIteratorStart;
					Char			*fileExtName	= NULL;

					//ȡָ
					while (volIterator != vfsIteratorStop)
					{
						VFSVolumeEnumerate(&volRef, &volIterator);
					}

					if (volRef > 0)
					{
						if (VFSFileOpen(volRef, "/PALM/Launcher", vfsModeReadWrite, &dirRef) == errNone)	// /PALM/Launcher/
						{
							fileInfo.nameP = nameP;
							fileInfo.nameBufLen = 128;

							while (dirIterator != vfsIteratorStop)
							{
								if (VFSDirEntryEnumerate(dirRef, &dirIterator, &fileInfo) == errNone)
								{
									fileExtName = strPreventChr(fileInfo.nameP, '.');	//ȡչ
									if (fileExtName != NULL)	//չ
									{
										if (StrNCaselessCompare(fileExtName, ".prc", 4) == 0)	//.prcΪһ
										{
											StrCopy(globeP->cacheP, "/PALM/Launcher/");
											StrCat(globeP->cacheP, fileInfo.nameP);
											if (VFSFileOpen(volRef, globeP->cacheP, vfsModeReadWrite, &fileRef) == errNone)
											{
												//س
												if (VFSFileDBInfo(fileRef, dbInfo.programNameP, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &type, &creatorId, NULL) == errNone)
												{
													if (type == 'appl' && (! (attributes & dmHdrAttrHidden)))
													{
														dbInfo.cardNo = 0; //not available
														dbInfo.dbId = 0;  //not available
														dbInfo.type = type;
														dbInfo.creatorId = creatorId;

														//ȡϢ
														getProgramCategory(&dbInfo);

														dbInfo.shortcut = chrNull;
														dbInfo.addOnInfo = 0;
														
														//ļ
														StrCopy(dbInfo.VFSNameP, fileInfo.nameP);

														//
														recordId = dmMaxRecordIndex;
														recordH = DmNewRecord(dbRef, &recordId, sizeof(ProgramListType));
														if (recordH != NULL)
														{
															recordP = (ProgramListType *)MemHandleLock(recordH);
															if (recordP != NULL)
																DmWrite(recordP, 0, &dbInfo, sizeof(ProgramListType));
															MemHandleUnlock(recordH);
															DmReleaseRecord(dbRef, recordId, true);
														}
													}
												}
												VFSFileClose(fileRef);
											}
										}
									}
								}
							}

							VFSFileClose(dirRef);
						}
						else	// /PALM/Launcher/ ڣ
						{
							VFSDirCreate(volRef, "/PALM/Launcher");
						}
					}
				}
				
				MemHandleUnlock(dbListH);
				MemHandleFree(dbListH);

				//ǰĬ4
				if (getProgramId(dbRef, MEMO_CREATOR, &recordId) == errNone)	//±
				{
					DmMoveRecord(dbRef, recordId, 0);
				}
				if (getProgramId(dbRef, CALCULATOR_CREATOR, &recordId) == errNone)	//ʼ
				{
					DmMoveRecord(dbRef, recordId, 0);
				}
				if (getProgramId(dbRef, BLAZER_CREATOR, &recordId) == errNone)	//
				{
					DmMoveRecord(dbRef, recordId, 0);
				}
				if (getProgramId(dbRef, CONTACTS_CREATOR, &recordId) == errNone)	//ͨѶ¼
				{
					DmMoveRecord(dbRef, recordId, 0);
				}
				else if (getProgramId(dbRef, ADDRESS_CREATOR, &recordId) == errNone)
				{
					DmMoveRecord(dbRef, recordId, 0);
				}

				DmCloseDatabase(dbRef);
			}
			break;
			
		case BUILD_LIST_MODE_REFRESH:
			//򿪳б
			dbRef = openProgramDatabase();

			//беļ¼Ƿ
			i = DmNumRecordsInCategory(dbRef, dmAllCategories);
			for (recordId = 0; recordId < i; recordId ++)
			{
				recordH = DmQueryRecord(dbRef, recordId);
				recordP = (ProgramListType *)MemHandleLock(recordH);
				//ڴ
				if (recordP->VFSNameP[0] == '\0')
				{
					if (DmGetNextDatabaseByTypeCreator(true, &dmSearchState, recordP->type, recordP->creatorId, true, &dbInfo.cardNo, &dbInfo.dbId) != dmErrCantFind)
					{
						if (DmDatabaseInfo(dbInfo.cardNo, dbInfo.dbId, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == errNone)
						{
							if ((attributes & dmHdrAttrHidden))	//أɾ
							{
								MemHandleUnlock(recordH);
								DmRemoveRecord(dbRef, recordId);
								i --;
								recordId --;
							}
							else
							{
								MemHandleUnlock(recordH);
							}
						}
						else	//޷ȡϢɾ
						{
							MemHandleUnlock(recordH);
							DmRemoveRecord(dbRef, recordId);
							i --;
							recordId --;
						}
					}
					else	//򲻴ڣɾ
					{
						MemHandleUnlock(recordH);
						DmRemoveRecord(dbRef, recordId);
						i --;
						recordId --;
					}
				}
				else if (updateCardPrograms)
				{
					UInt32			type;
					UInt32			creatorId;
					FileRef			fileRef;
					UInt16			volRef			= 0;
					UInt32			dirRef			= 0;
					UInt32			volIterator		= vfsIteratorStart;

					//ȡָ
					while (volIterator != vfsIteratorStop)
					{
						VFSVolumeEnumerate(&volRef, &volIterator);
					}

					if (volRef > 0)
					{
						if (VFSFileOpen(volRef, "/PALM/Launcher", vfsModeReadWrite, &dirRef) == errNone)	// /PALM/Launcher/
						{
							//·
							StrCopy(globeP->cacheP, "/PALM/Launcher/");
							StrCat(globeP->cacheP, recordP->VFSNameP);
							if (VFSFileOpen(volRef, globeP->cacheP, vfsModeReadWrite, &fileRef) == errNone)
							{
								if (VFSFileDBInfo(fileRef, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &type, &creatorId, NULL) == errNone)
								{
									if (attributes & dmHdrAttrHidden)	//أɾ
									{
										MemHandleUnlock(recordH);
										DmRemoveRecord(dbRef, recordId);
										i --;
										recordId --;
									}
									else
									{
										MemHandleUnlock(recordH);
									}
								}	//ܻȡϢɾ
								else
								{
									MemHandleUnlock(recordH);
									DmRemoveRecord(dbRef, recordId);
									i --;
									recordId --;
								}
								VFSFileClose(fileRef);
							}
							else //򲻿ɾ
							{
								MemHandleUnlock(recordH);
								DmRemoveRecord(dbRef, recordId);
								i --;
								recordId --;
							}
							VFSFileClose(dirRef);
						}
						else //·ڣɾ
						{
							MemHandleUnlock(recordH);
							DmRemoveRecord(dbRef, recordId);
							i --;
							recordId --;
						}
					}
					else //ڣɾ
					{
						MemHandleUnlock(recordH);
						DmRemoveRecord(dbRef, recordId);
						i --;
						recordId --;
					}
				}//ϳ
			}

			//ȡڴеĳб
			SysCreateDataBaseList('appl', 0, &dbListCount, &dbListH, true);
			dbListP = (SysDBListItemType *)MemHandleLock(dbListH);

			//Ƿµĳ
			for (i = 0; i < dbListCount; i ++)
			{
				if (dbListP[i].creator != APPLICATION_CREATOR && dbListP[i].creator != sysFileCLauncher/* && dbListP[i].creator != PREF_CREATOR*/)
				{
					if (DmDatabaseInfo(dbListP[i].cardNo, dbListP[i].dbID, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == errNone)
					{
						if ((! (attributes & dmHdrAttrHidden)))
						{
							if (! isDbInDbList(dbRef, dbListP[i].creator, NULL, NULL))
							{
								dbInfo.cardNo = dbListP[i].cardNo;
								dbInfo.dbId = dbListP[i].dbID;
								dbInfo.type = dbListP[i].type;
								dbInfo.creatorId = dbListP[i].creator;
								
								//ļ
								StrCopy(dbInfo.programNameP, dbListP[i].name);

								//ȡϢ
								getProgramCategory(&dbInfo);

								dbInfo.shortcut = chrNull;
								dbInfo.addOnInfo = 0;
								dbInfo.VFSNameP[0] = '\0';

								//
								recordId = dmMaxRecordIndex;
								recordH = DmNewRecord(dbRef, &recordId, sizeof(ProgramListType));
								recordP = (ProgramListType *)MemHandleLock(recordH);
								DmWrite(recordP, 0, &dbInfo, sizeof(ProgramListType));
								MemHandleUnlock(recordH);
								DmReleaseRecord(dbRef, recordId, true);
							}
						}
					}
				}
			}

			//鿨ϵ³
			if (updateCardPrograms)
			{
				FileInfoType	fileInfo;
				FileRef			fileRef;
				UInt32			type;
				UInt32			creatorId;
				UInt16			attributes;
				UInt16			volRef			= 0;
				UInt32			dirRef			= 0;
				UInt32			volIterator		= vfsIteratorStart;
				UInt32			dirIterator		= vfsIteratorStart;
				Char			*fileExtName	= NULL;

				//ȡָ
				while (volIterator != vfsIteratorStop)
				{
					VFSVolumeEnumerate(&volRef, &volIterator);
				}

				if (volRef > 0)
				{
					if (VFSFileOpen(volRef, "/PALM/Launcher", vfsModeReadWrite, &dirRef) == errNone)	// /PALM/Launcher/
					{
						fileInfo.nameP = nameP;
						fileInfo.nameBufLen = 128;

						while (dirIterator != vfsIteratorStop)
						{
							if (VFSDirEntryEnumerate(dirRef, &dirIterator, &fileInfo) == errNone)
							{
								fileExtName = strPreventChr(fileInfo.nameP, '.');	//ȡչ
								if (fileExtName != NULL)	//չ
								{
									if (StrNCaselessCompare(fileExtName, ".prc", 4) == 0)	//.prcΪһ
									{
										StrCopy(globeP->cacheP, "/PALM/Launcher/");
										StrCat(globeP->cacheP, fileInfo.nameP);
										if (VFSFileOpen(volRef, globeP->cacheP, vfsModeReadWrite, &fileRef) == errNone)
										{
											//س
											if (VFSFileDBInfo(fileRef, dbInfo.programNameP, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &type, &creatorId, NULL) == errNone)
											{
												if (type == 'appl' && (! (attributes & dmHdrAttrHidden)))
												{
													if (! isDbInDbList(dbRef, creatorId, fileInfo.nameP, NULL))
													{
														dbInfo.cardNo = 0; //not available
														dbInfo.dbId = 0;  //not available
														dbInfo.type = type;
														dbInfo.creatorId = creatorId;
														
														//ȡϢ
														getProgramCategory(&dbInfo);

														dbInfo.shortcut = chrNull;
														dbInfo.addOnInfo = 0;
														
														//ļ
														StrCopy(dbInfo.VFSNameP, fileInfo.nameP);

														//
														recordId = dmMaxRecordIndex;
														recordH = DmNewRecord(dbRef, &recordId, sizeof(ProgramListType));
														recordP = (ProgramListType *)MemHandleLock(recordH);
														DmWrite(recordP, 0, &dbInfo, sizeof(ProgramListType));
														MemHandleUnlock(recordH);
														DmReleaseRecord(dbRef, recordId, true);
													}
												}
											}
											VFSFileClose(fileRef);
										}
									}
								}
							}
						}

						VFSFileClose(dirRef);
					}
					else	// /PALM/Launcher/ ڣ
					{
						VFSDirCreate(volRef, "/PALM/Launcher");
					}
				}
			}//ϵ³

			MemHandleUnlock(dbListH);
			MemHandleFree(dbListH);

			DmCloseDatabase(dbRef);

			break;
			
		case BUILD_LIST_MODE_MOUNT:
			//򿪳б
			dbRef = openProgramDatabase();

			//ȡϵĳ
			{
				FileInfoType	fileInfo;
				FileRef			fileRef;
				UInt32			type;
				UInt32			creatorId;
				UInt16			attributes;
				UInt16			volRef			= 0;
				UInt32			dirRef			= 0;
				UInt32			volIterator		= vfsIteratorStart;
				UInt32			dirIterator		= vfsIteratorStart;
				Char			*fileExtName	= NULL;

				//ȡָ
				while (volIterator != vfsIteratorStop)
				{
					VFSVolumeEnumerate(&volRef, &volIterator);
				}

				if (volRef > 0)
				{
					if (VFSFileOpen(volRef, "/PALM/Launcher", vfsModeReadWrite, &dirRef) == errNone)	// /PALM/Launcher/
					{
						fileInfo.nameP = nameP;
						fileInfo.nameBufLen = 128;

						while (dirIterator != vfsIteratorStop)
						{
							if (VFSDirEntryEnumerate(dirRef, &dirIterator, &fileInfo) == errNone)
							{
								fileExtName = strPreventChr(fileInfo.nameP, '.');	//ȡչ
								if (fileExtName != NULL)	//չ
								{
									if (StrNCaselessCompare(fileExtName, ".prc", 4) == 0)	//.prcΪһ
									{
										StrCopy(globeP->cacheP, "/PALM/Launcher/");
										StrCat(globeP->cacheP, fileInfo.nameP);
										if (VFSFileOpen(volRef, globeP->cacheP, vfsModeReadWrite, &fileRef) == errNone)
										{
											//س
											if (VFSFileDBInfo(fileRef, dbInfo.programNameP, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &type, &creatorId, NULL) == errNone)
											{
												if (type == 'appl' && (! (attributes & dmHdrAttrHidden)))
												{
													dbInfo.cardNo = 0; //not available
													dbInfo.dbId = 0;  //not available
													dbInfo.type = type;
													dbInfo.creatorId = creatorId;

													//ȡϢ
													getProgramCategory(&dbInfo);

													dbInfo.shortcut = chrNull;
													dbInfo.addOnInfo = 0;
													
													//ļ
													StrCopy(dbInfo.VFSNameP, fileInfo.nameP);

													//
													recordId = dmMaxRecordIndex;
													recordH = DmNewRecord(dbRef, &recordId, sizeof(ProgramListType));
													recordP = (ProgramListType *)MemHandleLock(recordH);
													DmWrite(recordP, 0, &dbInfo, sizeof(ProgramListType));
													MemHandleUnlock(recordH);
													DmReleaseRecord(dbRef, recordId, true);
												}
											}
											VFSFileClose(fileRef);
										}
									}
								}
							}
						}

						VFSFileClose(dirRef);
					}
					else	// /PALM/Launcher/ ڣ
					{
						VFSDirCreate(volRef, "/PALM/Launcher");
					}
				}
			}
			
			DmCloseDatabase(dbRef);

			break;
			
		case BUILD_LIST_MODE_UNMOUNT:
			//򿪳б
			dbRef = openProgramDatabase();

			//беļ¼Ƿ
			i = DmNumRecordsInCategory(dbRef, dmAllCategories);
			for (recordId = 0; recordId < i; recordId ++)
			{
				recordH = DmQueryRecord(dbRef, recordId);
				recordP = (ProgramListType *)MemHandleLock(recordH);
				//
				if (recordP->VFSNameP[0] != '\0')
				{
					MemHandleUnlock(recordH);
					DmRemoveRecord(dbRef, recordId);
					i --;
					recordId --;
				}
				else
				{
					MemHandleUnlock(recordH);
				}
			}
			
			DmCloseDatabase(dbRef);
			break;

		/*case BUILD_LIST_MODE_ADD_DB:
			if (((SysNotifyDBCreatedType *)paramP)->type == 'appl' && ((SysNotifyDBCreatedType *)paramP)->creator != APPLICATION_CREATOR && ((SysNotifyDBCreatedType *)paramP)->creator != sysFileCLauncher)
			{
				if (DmGetNextDatabaseByTypeCreator(true, &dmSearchState, ((SysNotifyDBCreatedType *)paramP)->type, ((SysNotifyDBCreatedType *)paramP)->creator, true, &dbInfo.cardNo, &dbInfo.dbId) != dmErrCantFind)
				{
					if (DmDatabaseInfo(dbInfo.cardNo, dbInfo.dbId, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == errNone)
					{
						//򿪳б
						dbRef = openProgramDatabase();

						if ((! (attributes & dmHdrAttrHidden)))
						{
							//ǷѾڳб
							if (isDbInDbList(dbRef, ((SysNotifyDBCreatedType *)paramP)->creator, &recordId))
							{
								//ڣɾ
								DmRemoveRecord(dbRef, recordId);
							}

							//ӵб
							dbInfo.type = ((SysNotifyDBCreatedType *)paramP)->type;
							dbInfo.creatorId = ((SysNotifyDBCreatedType *)paramP)->creator;

							//ȡϢ
							getProgramCategory(&dbInfo);

							dbInfo.shortcut = chrNull;
							dbInfo.addOnInfo = 0;

							//
							recordId = dmMaxRecordIndex;
							recordH = DmNewRecord(dbRef, &recordId, sizeof(ProgramListType));
							recordP = (ProgramListType *)MemHandleLock(recordH);
							DmWrite(recordP, 0, &dbInfo, sizeof(ProgramListType));
							MemHandleUnlock(recordH);
							DmReleaseRecord(dbRef, recordId, true);
						}

						DmCloseDatabase(dbRef);
					}
				}
			}
			break;
		case BUILD_LIST_MODE_CHANGE_DB:
			if (((SysNotifyDBChangedType *)paramP)->oldCreator != APPLICATION_CREATOR && ((SysNotifyDBChangedType *)paramP)->oldCreator != sysFileCLauncher)
			{
				if ((((SysNotifyDBChangedType *)paramP)->type != 0 && (((SysNotifyDBChangedType *)paramP)->type != ((SysNotifyDBChangedType *)paramP)->oldType)) ||	//ͷ
						(((SysNotifyDBChangedType *)paramP)->oldType == 'appl' &&
							(((SysNotifyDBChangedType *)paramP)->creator != 0 && (((SysNotifyDBChangedType *)paramP)->creator != ((SysNotifyDBChangedType *)paramP)->oldCreator)) ||	//߷
							((((SysNotifyDBChangedType *)paramP)->oldAttributes & dmHdrAttrHidden) || (((SysNotifyDBChangedType *)paramP)->attributes & dmHdrAttrHidden))	//Խ޸
						)
					)
				{
					//򿪳б
					dbRef = openProgramDatabase();

					//беļ¼Ƿ
					i = DmNumRecordsInCategory(dbRef, dmAllCategories);
					for (recordId = 0; recordId < i; recordId ++)
					{
						recordH = DmQueryRecord(dbRef, recordId);
						recordP = (ProgramListType *)MemHandleLock(recordH);
						if (DmGetNextDatabaseByTypeCreator(true, &dmSearchState, recordP->type, recordP->creatorId, true, &dbInfo.cardNo, &dbInfo.dbId) != dmErrCantFind)
						{
							if (DmDatabaseInfo(dbInfo.cardNo, dbInfo.dbId, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == errNone)
							{
								if ((attributes & dmHdrAttrHidden))	//أɾ
								{
									MemHandleUnlock(recordH);
									DmRemoveRecord(dbRef, recordId);
									i --;
									recordId --;
								}
								else
								{
									MemHandleUnlock(recordH);
								}
							}
							else	//޷ȡϢɾ
							{
								MemHandleUnlock(recordH);
								DmRemoveRecord(dbRef, recordId);
								i --;
								recordId --;
							}
						}
						else	//򲻴ڣɾ
						{
							MemHandleUnlock(recordH);
							DmRemoveRecord(dbRef, recordId);
							i --;
							recordId --;
						}
					}

					//ȡڴеĳб
					SysCreateDataBaseList('appl', 0, &dbListCount, &dbListH, true);
					dbListP = (SysDBListItemType *)MemHandleLock(dbListH);

					//Ƿµĳ
					for (i = 0; i < dbListCount; i ++)
					{
						if (dbListP[i].creator != APPLICATION_CREATOR && dbListP[i].creator != sysFileCLauncher)
						{
							if (DmDatabaseInfo(dbListP[i].cardNo, dbListP[i].dbID, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == errNone)
							{
								if ((! (attributes & dmHdrAttrHidden)))
								{
									if (! isDbInDbList(dbRef, dbListP[i].creator, NULL))
									{
										dbInfo.cardNo = dbListP[i].cardNo;
										dbInfo.dbId = dbListP[i].dbID;
										dbInfo.type = dbListP[i].type;
										dbInfo.creatorId = dbListP[i].creator;

										//ȡϢ
										getProgramCategory(&dbInfo);

										dbInfo.shortcut = chrNull;
										dbInfo.addOnInfo = 0;

										//
										recordId = dmMaxRecordIndex;
										recordH = DmNewRecord(dbRef, &recordId, sizeof(ProgramListType));
										recordP = (ProgramListType *)MemHandleLock(recordH);
										DmWrite(recordP, 0, &dbInfo, sizeof(ProgramListType));
										MemHandleUnlock(recordH);
										DmReleaseRecord(dbRef, recordId, true);
									}
								}
							}
						}
					}
					MemHandleUnlock(dbListH);
					MemHandleFree(dbListH);

					DmCloseDatabase(dbRef);
				}
			}
			break;
		case BUILD_LIST_MODE_DELETE_DB:
			if (((SysNotifyDBDeletedType *)paramP)->type == 'appl' && ((SysNotifyDBDeletedType *)paramP)->creator != APPLICATION_CREATOR &&
				((SysNotifyDBDeletedType *)paramP)->creator != sysFileCLauncher &&
				(! (((SysNotifyDBDeletedType *)paramP)->attributes & dmHdrAttrHidden)))
			{
				//򿪳б
				dbRef = openProgramDatabase();

				if (isDbInDbList(dbRef, ((SysNotifyDBDeletedType *)paramP)->creator, &recordId))
				{
					//ڣɾ
					DmRemoveRecord(dbRef, recordId);
				}

				DmCloseDatabase(dbRef);
			}
			break;*/
	}
}

//ȡָΧĳϢ
static void getProgramListInCategory(UInt8 categoryId)
{
	UInt32				i;
	UInt16				endIndex;
	DmOpenRef			dbRef;
	MemHandle			recordH;
	ProgramListType		*recordP;

	//ճб
	freeGlobeProgramList();

	//ڴеĳ
	if (categoryId != CARD_CATEGORY)
	{
		//ݿ
		dbRef = DmOpenDatabaseByTypeCreator(PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadOnly);

		if (dbRef != NULL)
		{
			//ȡΧ
			endIndex = DmNumRecordsInCategory(dbRef, dmAllCategories);

			//ڴ
			i = sizeof(ProgramListType) * endIndex;
			globeP->programListP = (ProgramListType *)MemPtrNew(i);
			MemSet(globeP->programListP, i, 0);

			//
			if (categoryId != ALL_CATEGORIES)
			{
				for (i = 0; i < endIndex; i ++)
				{
					recordH = DmQueryRecord(dbRef, i);
					recordP = (ProgramListType *)MemHandleLock(recordH);
					
					//ûпԹϵĳ
					if
					(
						recordP->categoryId == categoryId
						&&
						(
							globeP->isVolumeMounted
							||
							recordP->VFSNameP[0] == '\0'
						)
					)
					{
						globeP->programListP[globeP->programListCount] = (*recordP);
						globeP->programListCount ++;
					}
					MemHandleUnlock(recordH);
				}
			}
			else
			{
				for (i = 0; i < endIndex; i ++)
				{
					recordH = DmQueryRecord(dbRef, i);
					recordP = (ProgramListType *)MemHandleLock(recordH);
					if
					(
						!isCategoryHide(recordP->categoryId)
						&&
						(
							globeP->isVolumeMounted
							||
							recordP->VFSNameP[0] == '\0'
						)
					)
					{
						globeP->programListP[globeP->programListCount] = (*recordP);
						globeP->programListCount ++;
					}
					MemHandleUnlock(recordH);
				}
			}

			//رݿ
			DmCloseDatabase(dbRef);
		}
	}
}

//ڸķм
static void searchProgramInCategory(Char *key, UInt8 categoryId/*, SettingType *settingP*/)
{
	UInt32				i;
	UInt16				endIndex;
	UInt8				j;
	Boolean				isMatched;
	DmOpenRef			dbRef;
	MemHandle			recordH;
	ProgramListType		*recordP;
	UInt32				ctx;
	UInt32				*pyTableP;
	UInt16				UInt32KeyLength;
	UInt16				UInt32NameLength;
	UInt16				contactSearchLibRef	= 0;
	UInt32				*UInt32KeyP			= (UInt32 *)MemPtrNew(24);
	UInt32				*UInt32NameP		= (UInt32 *)MemPtrNew(24);

	//Դƴ⣬ƴת
	if (CLibLoad(&contactSearchLibRef, &ctx) == errNone)
	{
		if (FtrGet(CLibCreatorID, PY_TABLE_FTR_NUMBER, (UInt32 *)&pyTableP) != errNone)
		{
			pyTableP = NULL;
		}
	}
	else
	{
		pyTableP = NULL;
	}

	//ѼؼתUInt32λװ
	convertCharsToBitPackage(key, pyTableP, UInt32KeyP, &UInt32KeyLength);

	//ʼճб
	freeGlobeProgramList();

	//ݿ
	dbRef = DmOpenDatabaseByTypeCreator(PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadOnly);

	//ȡΧ
	endIndex = DmNumRecordsInCategory(dbRef, dmAllCategories);

	//ڴ
	i = sizeof(ProgramListType) * endIndex;
	globeP->programListP = (ProgramListType *)MemPtrNew(i);
	MemSet(globeP->programListP, i, 0);

	//
	if (categoryId != ALL_CATEGORIES)
	{
		for (i = 0; i < endIndex; i ++)
		{
			recordH = DmQueryRecord(dbRef, i);
			recordP = (ProgramListType *)MemHandleLock(recordH);
			if (recordP->categoryId == categoryId)
			{
				//תUInt32λװ
				convertCharsToBitPackage(recordP->programNameP, pyTableP, UInt32NameP, &UInt32NameLength);

				//Ƚ
				isMatched = true;
				for (j = 0; j < UInt32KeyLength; j ++)
				{
					if (j >= UInt32NameLength)
					{
						isMatched = false;
						break;
					}
					else if (! (UInt32KeyP[j] & UInt32NameP[j]))
					{
						isMatched = false;
						break;
					}
				}

				if (isMatched)
				{
					globeP->programListP[globeP->programListCount] = (*recordP);
					globeP->programListCount ++;
				}
			}
			MemHandleUnlock(recordH);
		}
	}
	else
	{
		for (i = 0; i < endIndex; i ++)
		{
			recordH = DmQueryRecord(dbRef, i);
			recordP = (ProgramListType *)MemHandleLock(recordH);
			if (! isCategoryHide(recordP->categoryId))
			{
				//ֲתUInt32λװ
				convertCharsToBitPackage(recordP->programNameP, pyTableP, UInt32NameP, &UInt32NameLength);

				//Ƚ
				isMatched = true;
				for (j = 0; j < UInt32KeyLength; j ++)
				{
					if (j >= UInt32NameLength)
					{
						isMatched = false;
						break;
					}
					else if (! (UInt32KeyP[j] & UInt32NameP[j]))
					{
						isMatched = false;
						break;
					}
				}

				if (isMatched)
				{
					globeP->programListP[globeP->programListCount] = (*recordP);
					globeP->programListCount ++;
				}
			}
			MemHandleUnlock(recordH);
		}
	}

	//رݿ
	DmCloseDatabase(dbRef);

	//رƴ
	if (pyTableP != NULL)
	{
		CLibRemove(contactSearchLibRef, ctx);
	}

	MemPtrFree(UInt32NameP);
	MemPtrFree(UInt32KeyP);
}

//ȡơͼͿݼ
static void getProgramItemInfo(UInt8 deviceModel ,ProgramListType *programP, ProgramItemInfoType *programInfoP)
{
	MemHandle		iconRscH;
	MemPtr 			iconRscP;
	UInt16			bmpIconSize;

	//ʼֹγ
	programInfoP->bmpIconP = NULL;
	programInfoP->nameP[0] = '-';
	programInfoP->nameP[1] = '\0';
	

	if (programP->VFSNameP[0] == '\0')	//ڴ
	{
		DmOpenRef		dbRef			= DmOpenDatabaseByTypeCreator('appl', programP->creatorId, dmModeReadOnly);

		if (dbRef != NULL)
		{
			//
			if (programP->programNameP[0] != '\0')
			{
				StrCopy(programInfoP->nameP, programP->programNameP);
			}

			//ͼ
			iconRscH = DmGet1Resource('tAIB', 1000);
			if (iconRscH)
			{
				bmpIconSize = MemHandleSize(iconRscH);
				programInfoP->bmpIconP = (BitmapType *)MemPtrNew(bmpIconSize);
				if (programInfoP->bmpIconP != NULL)
				{
					iconRscP = MemHandleLock(iconRscH);
					if (iconRscP != NULL)
					{
						MemMove(programInfoP->bmpIconP, iconRscP, bmpIconSize);
						MemHandleUnlock(iconRscH);
					}
				}
				DmReleaseResource(iconRscH);
				DmCloseDatabase(dbRef);
			}
			else	//ûͼ
			{
				DmCloseDatabase(dbRef);
				dbRef = DmOpenDatabaseByTypeCreator('appl', APPLICATION_CREATOR, dmModeReadOnly);
				if (dbRef != NULL)
				{
					if (deviceModel != TREO_600)	//߷
					{
						iconRscH = DmGet1Resource('Tbmp', bmpNoIconLarge);
					}
					else	//ͷ
					{
						iconRscH = DmGet1Resource('Tbmp', bmpNoIconSmall);
					}
					if (iconRscH)
					{
						bmpIconSize = MemHandleSize(iconRscH);
						programInfoP->bmpIconP = (BitmapType *)MemPtrNew(bmpIconSize);
						
						if (programInfoP->bmpIconP != NULL)
						{

							iconRscP = MemHandleLock(iconRscH);
							if (iconRscP != NULL)
							{
								MemMove(programInfoP->bmpIconP, iconRscP, bmpIconSize);
								MemHandleUnlock(iconRscH);
							}
						}
						DmReleaseResource(iconRscH);
					}
					DmCloseDatabase(dbRef);
				}
			}
		}
	}
	else	//ڿ
	{
		FileRef			fileRef;
		UInt16			volRef			= 0;
		UInt32			dirRef			= 0;
		UInt32			volIterator		= vfsIteratorStart;

		//ȡָ
		while (volIterator != vfsIteratorStop)
		{
			VFSVolumeEnumerate(&volRef, &volIterator);
		}

		if (volRef > 0)
		{
			if (VFSFileOpen(volRef, "/PALM/Launcher", vfsModeReadWrite, &dirRef) == errNone)	// /PALM/Launcher/
			{
				//·
				StrCopy(globeP->cacheP, "/PALM/Launcher/");
				StrCat(globeP->cacheP, programP->VFSNameP);
				
				//򿪳ȡԴ
				if (VFSFileOpen(volRef, globeP->cacheP, vfsModeReadWrite, &fileRef) == errNone)
				{
					//ȡ
					if (programP->programNameP[0] != '\0')
					{
						StrCopy(programInfoP->nameP, programP->programNameP);
					}
					else if (programP->VFSNameP[0] != '\0')
					{
						StrCopy(programInfoP->nameP, programP->VFSNameP);
					}

					//ȡͼ
					if (VFSFileDBGetResource(fileRef, 'tAIB', 1000, &iconRscH) == errNone)
					{
						bmpIconSize = MemHandleSize(iconRscH);
						programInfoP->bmpIconP = (BitmapType *)MemPtrNew(bmpIconSize);
						if (programInfoP->bmpIconP != NULL)
						{
							iconRscP = MemHandleLock(iconRscH);
							if (iconRscP != NULL)
							{
								MemMove(programInfoP->bmpIconP, iconRscP, bmpIconSize);
								MemHandleUnlock(iconRscH);
							}
						}	
						MemHandleFree(iconRscH);
					}
					else	//ûͼ
					{
						DmOpenRef		dbRef	= DmOpenDatabaseByTypeCreator('appl', APPLICATION_CREATOR, dmModeReadOnly);

						if (deviceModel != TREO_600)	//߷
						{
							iconRscH = DmGet1Resource('Tbmp', bmpNoIconLarge);
						}
						else	//ͷ
						{
							iconRscH = DmGet1Resource('Tbmp', bmpNoIconSmall);
						}
						bmpIconSize = MemHandleSize(iconRscH);
						programInfoP->bmpIconP = (BitmapType *)MemPtrNew(bmpIconSize);
						
						if (programInfoP->bmpIconP != NULL)
						{
							iconRscP = MemHandleLock(iconRscH);
							if (iconRscP != NULL)
							{
								MemMove(programInfoP->bmpIconP, iconRscP, bmpIconSize);
								MemHandleUnlock(iconRscH);
							}
						}	
						DmReleaseResource(iconRscH);
						DmCloseDatabase(dbRef);
					}
					VFSFileClose(fileRef);
				}


				VFSFileClose(dirRef);
			}
			else	// /PALM/Launcher/ ڣ
			{
				VFSDirCreate(volRef, "/PALM/Launcher");
			}
		}
	}
}

#if 0
#pragma mark -
#pragma mark LAUNCHER FUNCTIONS
#pragma mark -
#endif

#if 0
#pragma mark == UI FUNCTIONS ==
#endif

//ȡǰ״̬
static UInt8 getToneStatus(Boolean *isVibrateON)
{
	SoundPreference		soundPrefs;
	UInt32				ringStatus;

	HsAttrGet(hsAttrRingSwitch, 0, &ringStatus);
	TonesLibGetSoundPrefs(globeP->tonesLibRef, soundPrefTypePhone, &soundPrefs);

	if (ringStatus != hsAttrRingSwitchMute)
	{
		(*isVibrateON) = soundPrefs.soundOnVibrate;
	}
	else
	{
		(*isVibrateON) = soundPrefs.soundOffVibrate;
		soundPrefs.soundOnVolume = 0;
	}


	return soundPrefs.soundOnVolume;
}

/*
For CDMA, the number returned is the ABS(dBm). A value of 113 corresponds to
-113dBm.

According to GSM 07.07 Section 8.5, convert from dBm to rssi [0..31]:

0       -113 dBm or less
1       -111 dBm
2...30  -109... -53 dBm   [ (113 - s)/2 ]
31      -51 dBm or greater

To convert from rssi to bars [0..5]

0       -113 dBm or less
1       -111 dBm
2...4   -109... -53 dBm   [ ((rssi * 2)/15 + 1 ]
5       -51 dBm or greater


For GSM, I believe the number returned is rssi [0..31].

0       0..3
1..4    4..24   [ rssi/6 + 1 ]
5       25..n
*/
//ȡǰźǿȣ0-4ֵ
//㹫ʽ趨16Ϊ4
static UInt8 getSignalLevel()
{
	UInt16 phoneSignalQuality = 0;

	PhnLibSignalQuality(globeP->hsPhoneLibRef, &phoneSignalQuality);
	
	if (phoneSignalQuality <= 13)
		return (UInt8)(phoneSignalQuality / 4);
	else
		return 4;
}

//ȡǰʣذٷֱ
static UInt8 getBatteryPercentage(Boolean *pluggedIn)
{
	UInt8	percentP = 0;

	SysBatteryInfo(false, NULL, NULL, NULL, NULL, pluggedIn, &percentP);

	return percentP;
}

//ʾز˵ؼѡ
static void showHideMenuitem()
{
	//UInt16		libRef;
	//Err			errorCode;
	if (globeP->isPhoneOn)	//绰Ѿ
	{
		MenuHideItem(menuitemPhoneON);
		MenuShowItem(menuitemPhoneOFF);
	}
	else
	{
		MenuShowItem(menuitemPhoneON);
		MenuHideItem(menuitemPhoneOFF);
	}

	/*
	//ǷѾ
	if (SysLibFind("Net.lib", &libRef) == errNone)
	{
		NetLibOpenCount(libRef, &errorCode);
		if (errorCode > 0)
		{
			MenuHideItem(menuitemConnectNetwork);
			MenuShowItem(menuitemDropNetwork);
		}
		else
		{
			MenuShowItem(menuitemConnectNetwork);
			MenuHideItem(menuitemDropNetwork);
		}
	}*/
}

//ָؼľ
static RectangleType *getFocusedObjectBounds(UInt16 objectId)
{
	switch (objectId)
	{
		case gdtCarrier:
			globeP->focusedObjectRectangle.topLeft.x = 2; globeP->focusedObjectRectangle.topLeft.y = 2;
			globeP->focusedObjectRectangle.extent.x = 80; globeP->focusedObjectRectangle.extent.y = 10;
			break;
		case gdtBlueTooth:
			globeP->focusedObjectRectangle.topLeft.x = 120; globeP->focusedObjectRectangle.topLeft.y = 1;
			globeP->focusedObjectRectangle.extent.x = 9; globeP->focusedObjectRectangle.extent.y = 10;
			break;
		case gdtSignal:
			globeP->focusedObjectRectangle.topLeft.x = 131; globeP->focusedObjectRectangle.topLeft.y = 1;
			globeP->focusedObjectRectangle.extent.x = 14; globeP->focusedObjectRectangle.extent.y = 10;
			break;
		case gdtBattery:
			globeP->focusedObjectRectangle.topLeft.x = 150; globeP->focusedObjectRectangle.topLeft.y = 1;
			globeP->focusedObjectRectangle.extent.x = 8; globeP->focusedObjectRectangle.extent.y = 10;
			break;
		case gdtDatetime:
			globeP->focusedObjectRectangle.topLeft.x = 2; globeP->focusedObjectRectangle.topLeft.y = 15;
			globeP->focusedObjectRectangle.extent.x = 156; globeP->focusedObjectRectangle.extent.y = 11;
			break;
		case ptriCategory:
			FrmGetObjectBounds(globeP->frmPhoneUIP, FrmGetObjectIndex(globeP->frmPhoneUIP, ptriCategory), &globeP->focusedObjectRectangle);
			break;
/*			
		case fldKeyword:
			globeP->focusedObjectRectangle.topLeft.x = 110; globeP->focusedObjectRectangle.topLeft.y = 148;
			globeP->focusedObjectRectangle.extent.x = 48; globeP->focusedObjectRectangle.extent.y = 11;
			break;
*/
	}

	return &globeP->focusedObjectRectangle;
}

//ȡʶ
static void getOperatorName(SettingType *settingP)
{
	if (PhnLibModulePowered(globeP->hsPhoneLibRef) == phnPowerOff)
	{
		StrCopy(globeP->operatorNameP, STRING_PHONE_OFF);
	}
	else
	{
		UInt32				E911Mode = true;

		HsAttrGet(hsAttrE911Mode, 0, &E911Mode);

		if ((Boolean)E911Mode == false)
		{
			if (settingP->useCustomCarrier)
			{
				StrCopy(globeP->operatorNameP, settingP->customCarrierP);
			}
			else
			{
				GSMRegistrationMode		gsmRegMode;
				PhnOperatorID			operatorID;
				Char					*bufferP	= NULL;

				PhnLibCurrentOperator(globeP->hsPhoneLibRef, &operatorID, &bufferP, &gsmRegMode);
				if (bufferP)
				{
					StrCopy(globeP->operatorNameP, bufferP);
					MemPtrFree(bufferP);
				}
			}
		}
		else
		{
			StrCopy(globeP->operatorNameP, STRING_SOS_CALL_ONLY);
		}
	}
}

//ȡͼ
static BitmapTypeV3 *getBackgdBitmap(SettingType *settingP)
{
	UInt16			error;
	FileHand		fileH;
	void			*bmpBitsP	= NULL;
	BitmapTypeV3	*bmpV3P		= NULL;

	//ͷžɵıͼ
	if (FtrGet(APPLICATION_CREATOR, BITMAP_BITS_FEATURE_ID, (UInt32 *)&bmpBitsP) == errNone)
	{
		FtrPtrFree(APPLICATION_CREATOR, BITMAP_BITS_FEATURE_ID);
	}

	//ȡͼ
	fileH = FileOpen(0, WALLPAPER_CACHE_NAME, sysFileTFileStream, APPLICATION_CREATOR, fileModeReadOnly, &error);
	if (error == errNone)
	{
		UInt32		width, height;
		Int32		bmpBitsSize;

		FileTell(fileH, &bmpBitsSize, &error);
		bmpBitsSize -= 8;
		FileRead(fileH, &width, 4, 1, &error);
		FileRead(fileH, &height, 4, 1, &error);

		if (error == errNone)
		{
			BitmapType	*bmpP	= BmpCreate(width, height, 16, NULL, &error);

			if (error == errNone)
			{
				if (FtrPtrNew(APPLICATION_CREATOR, BITMAP_BITS_FEATURE_ID, bmpBitsSize, &bmpBitsP) == errNone)
				{
					FileDmRead(fileH, bmpBitsP, 0, bmpBitsSize, 1, &error);

					if (error == errNone)
					{
						bmpV3P = BmpCreateBitmapV3(bmpP, kDensityDouble, bmpBitsP, NULL);
					}
					else
					{
						FtrPtrFree(APPLICATION_CREATOR, BITMAP_BITS_FEATURE_ID);
					}
				}
				BmpDelete(bmpP);
			}
		}

		FileClose(fileH);
	}

	return bmpV3P;
}

static void getSoundStatusBitmap(SettingType *settingP)
{
	UInt16			i;
	UInt16			bmpSize;
	MemHandle		bmpRscH;
	DmOpenRef		dbRef		= DmOpenDatabaseByTypeCreator('appl', APPLICATION_CREATOR, dmModeReadOnly);

	/*if (deviceModel != TREO_600)	//߷
	{
		iconRscH = DmGet1Resource('Tbmp', bmpNoIconLarge);
	}
	else	//ͷ
	{
		iconRscH = DmGet1Resource('Tbmp', bmpNoIconSmall);
	}*/
	if (settingP->deviceModel != TREO_600)
	{
		for (i = 0; i < 10; i ++)
		{
			bmpRscH = DmGet1Resource('Tbmp', i + bmpSpeakerON);
			bmpSize = MemHandleSize(bmpRscH);

			globeP->bmpSoundStatusP[i] = (BitmapType *)MemPtrNew(bmpSize);
			MemMove(globeP->bmpSoundStatusP[i], MemHandleLock(bmpRscH), bmpSize);

			MemHandleUnlock(bmpRscH);
			DmReleaseResource(bmpRscH);
		}

		for (i = 0; i < 2; i++)
		{
			bmpRscH = DmGet1Resource('Tbmp', bmpPowerCharge + i);
			if (bmpRscH != NULL)
			{
				bmpSize = MemHandleSize(bmpRscH);

				globeP->bmpPowerStatusP[i] = (BitmapType *)MemPtrNew(bmpSize);
				MemMove(globeP->bmpPowerStatusP[i], MemHandleLock(bmpRscH), bmpSize);

				MemHandleUnlock(bmpRscH);
				DmReleaseResource(bmpRscH);
			}
		}
	}
	else
	{
		for (i = 0; i < 10; i ++)
		{
			if (i < 2)
			{
				bmpRscH = DmGet1Resource('Tbmp', i + bmpSpeakerONs);
			}
			else if (i == 9)
			{
				bmpRscH = DmGet1Resource('Tbmp', bmpVibrates);
			}
			else if (i == 2 && i == 3)
			{
				bmpRscH = DmGet1Resource('Tbmp', bmpVolume1s);
			}
			else if (i == 4 && i == 5 && i == 6)
			{
				bmpRscH = DmGet1Resource('Tbmp', bmpVolume2s);
			}
			else
			{
				bmpRscH = DmGet1Resource('Tbmp', bmpVolume3s);
			}
			bmpSize = MemHandleSize(bmpRscH);

			globeP->bmpSoundStatusP[i] = (BitmapType *)MemPtrNew(bmpSize);
			MemMove(globeP->bmpSoundStatusP[i], MemHandleLock(bmpRscH), bmpSize);

			MemHandleUnlock(bmpRscH);
			DmReleaseResource(bmpRscH);
		}
	}

	DmCloseDatabase(dbRef);
}

static void setRectangleData()
{
	globeP->backgdRectangle.topLeft.x = 0;
	globeP->backgdRectangle.topLeft.y = 0;
	globeP->backgdRectangle.extent.x = 160;
	globeP->backgdRectangle.extent.y = 160;

	globeP->statusBarRectangle.topLeft.x = 0;
	globeP->statusBarRectangle.topLeft.y = 0;
	globeP->statusBarRectangle.extent.x = 160;
	globeP->statusBarRectangle.extent.y = 13;

	globeP->datetimeBarRectangle.topLeft.x = 0;
	globeP->datetimeBarRectangle.topLeft.y = 13;
	globeP->datetimeBarRectangle.extent.x = 160;
	globeP->datetimeBarRectangle.extent.y = 15;

	globeP->launchBarRectangle.topLeft.x = 0;
	globeP->launchBarRectangle.topLeft.y = 115;
	globeP->launchBarRectangle.extent.x = 160;
	globeP->launchBarRectangle.extent.y = 45;

	/*globeP->launchViewRectangle.topLeft.x = 0;
	globeP->launchViewRectangle.topLeft.y = 28;
	globeP->launchViewRectangle.extent.x = 160;
	globeP->launchViewRectangle.extent.y = 132;*/
	globeP->launchViewRectangle.topLeft.x = 0;
	globeP->launchViewRectangle.topLeft.y = 13;
	globeP->launchViewRectangle.extent.x = 160;
	globeP->launchViewRectangle.extent.y = 147;

	globeP->launchViewTitleBar.topLeft.x = 0;
	globeP->launchViewTitleBar.topLeft.y = 15;
	globeP->launchViewTitleBar.extent.x = 160;
	globeP->launchViewTitleBar.extent.y = 12;

	globeP->launchViewStatusBar.topLeft.x = 0;
	globeP->launchViewStatusBar.topLeft.y = 146;
	globeP->launchViewStatusBar.extent.x = 160;
	globeP->launchViewStatusBar.extent.y = 14;

	/*globeP->positionRectangle[0].topLeft.x = 0;
	globeP->positionRectangle[0].extent.x = 40;

	globeP->positionRectangle[1].topLeft.x = 40;
	globeP->positionRectangle[1].extent.x = 40;

	globeP->positionRectangle[2].topLeft.x = 80;
	globeP->positionRectangle[2].extent.x = 40;

	globeP->positionRectangle[3].topLeft.x = 120;
	globeP->positionRectangle[3].extent.x = 40;*/
}

#if 0
#pragma mark -----------------------------------------------------
#endif

//
static void drawIconFocusedRing(RectangleType *rectangleP, UInt8 style, SettingType *settingP)
{
	Int16		i, n;
	Coord		x;
	Coord		y;

	if (settingP->deviceModel != TREO_600)
	{
		switch (style)
		{
			case FOCUS_RING_STYLE_ROUND_RECTANGLE:
				HsNavDrawFocusRing(globeP->frmPhoneUIP, gdtIcons, hsNavFocusRingNoExtraInfo, rectangleP, hsNavFocusRingStyleRound, false);
				break;
			case FOCUS_RING_STYLE_SHARP_RECTANGLE:
				HsNavDrawFocusRing(globeP->frmPhoneUIP, gdtIcons, hsNavFocusRingNoExtraInfo, rectangleP, hsNavFocusRingStyleSquare, false);
				break;
			case FOCUS_RING_STYLE_HORIZONTAL_BARS:
				HsNavDrawFocusRing(globeP->frmPhoneUIP, gdtIcons, hsNavFocusRingNoExtraInfo, rectangleP, hsNavFocusRingStyleHorizontalBars, false);
				break;
			case FOCUS_RING_STYLE_BOTTOM_BAR:
				WinSetCoordinateSystem(kCoordinatesDouble);

				x = (rectangleP->topLeft.x - 3) * 2 + 3;
				y = (rectangleP->topLeft.y + rectangleP->extent.y - 1) * 2;
				n = (rectangleP->topLeft.x + rectangleP->extent.x + 3) * 2 - 3;

				for (i = 5; i >= 3; i --)
				{
					WinSetForeColorRGB(&settingP->focusRingColor[i], NULL);
					WinPaintLine(x, y, n, y);
					y ++;
					x --;
					n ++;
				}
				x ++;
				n --;
				for (i = 2; i >= 0; i --)
				{
					WinSetForeColorRGB(&settingP->focusRingColor[i], NULL);
					WinPaintLine(x, y, n, y);
					y ++;
					x ++;
					n --;
				}

				WinSetCoordinateSystem(kCoordinatesStandard);
				break;
			case FOCUS_RING_STYLE_ANGLE:
				WinSetCoordinateSystem(kCoordinatesDouble);

				{
					WinLineType		l[4][2];

					l[0][0].x1 = (rectangleP->topLeft.x - 3) * 2;
					l[0][0].y1 = (rectangleP->topLeft.y - 3) * 2;
					l[0][0].x2 = l[0][0].x1;
					l[0][0].y2 = l[0][0].y1 + 18;
					l[0][1].x1 = l[0][0].x1;
					l[0][1].y1 = l[0][0].y1;
					l[0][1].x2 = l[0][1].x1 + 12;
					l[0][1].y2 = l[0][1].y1;

					l[1][0].x1 = (rectangleP->topLeft.x + rectangleP->extent.x + 3) * 2;
					l[1][0].y1 = l[0][0].y1;
					l[1][0].x2 = l[1][0].x1;
					l[1][0].y2 = l[1][0].y1 + 18;
					l[1][1].x1 = l[1][0].x1;
					l[1][1].y1 = l[1][0].y1;
					l[1][1].x2 = l[1][1].x1 - 12;
					l[1][1].y2 = l[1][1].y1;

					l[2][0].x1 = l[0][0].x1;
					l[2][0].y1 = (rectangleP->topLeft.y + rectangleP->extent.y + 2) * 2;
					l[2][0].x2 = l[2][0].x1;
					l[2][0].y2 = l[2][0].y1 - 18;
					l[2][1].x1 = l[2][0].x1;
					l[2][1].y1 = l[2][0].y1;
					l[2][1].x2 = l[2][1].x1 + 12;
					l[2][1].y2 = l[2][1].y1;

					l[3][0].x1 = l[1][0].x1;
					l[3][0].y1 = l[2][0].y1;
					l[3][0].x2 = l[3][0].x1;
					l[3][0].y2 = l[3][0].y1 - 18;
					l[3][1].x1 = l[3][0].x1;
					l[3][1].y1 = l[3][0].y1;
					l[3][1].x2 = l[3][1].x1 - 12;
					l[3][1].y2 = l[3][1].y1;

					for (i = 5; i >= 0; i --)
					{
						WinSetForeColorRGB(&settingP->focusRingColor[i], NULL);

						for (n = 0; n < 4; n ++)
						{
							WinPaintLines(2, l[n]);
						}

						l[0][0].x1 ++;
						l[0][0].y1 ++;
						l[0][0].x2 ++;
						l[0][1].x1 ++;
						l[0][1].y1 ++;
						l[0][1].y2 ++;

						l[1][0].x1 --;
						l[1][0].y1 ++;
						l[1][0].x2 --;
						l[1][1].x1 --;
						l[1][1].y1 ++;
						l[1][1].y2 ++;

						l[2][0].x1 ++;
						l[2][0].y1 --;
						l[2][0].x2 ++;
						l[2][1].x1 ++;
						l[2][1].y1 --;
						l[2][1].y2 --;

						l[3][0].x1 --;
						l[3][0].y1 --;
						l[3][0].x2 --;
						l[3][1].x1 --;
						l[3][1].y1 --;
						l[3][1].y2 --;
					}
				}

				WinSetCoordinateSystem(kCoordinatesStandard);
				break;
			case FOCUS_RING_STYLE_ROUND_FRAME:
				rectangleP->topLeft.x --;
				rectangleP->topLeft.y --;
				rectangleP->extent.x += 2;
				rectangleP->extent.y += 2;
				WinSetForeColorRGB(&settingP->focusRingColor[0], NULL);
				WinPaintRectangleFrame(roundFrame, rectangleP);
				break;
			case FOCUS_RING_STYLE_FRMAE_WITH_MASK:
				rectangleP->topLeft.x --;
				rectangleP->topLeft.y --;
				rectangleP->extent.x += 2;
				rectangleP->extent.y += 2;
				WinSetForeColorRGB(&settingP->focusRingColor[3], NULL);
				WinPaintRectangle (rectangleP, 4);
				WinSetForeColorRGB(&settingP->focusRingColor[0], NULL);
				WinPaintRectangleFrame(roundFrame, rectangleP);
				break;
		}
	}
	else
	{
		switch (style)
		{
			case FOCUS_RING_STYLE_SHARP_RECTANGLE:
				HsNavDrawFocusRing(globeP->frmPhoneUIP, gdtIcons, hsNavFocusRingNoExtraInfo, rectangleP, hsNavFocusRingStyleSquare, false);
				break;
			case FOCUS_RING_STYLE_HORIZONTAL_BARS:
				HsNavDrawFocusRing(globeP->frmPhoneUIP, gdtIcons, hsNavFocusRingNoExtraInfo, rectangleP, hsNavFocusRingStyleHorizontalBars, false);
				break;
			default:
				HsNavDrawFocusRing(globeP->frmPhoneUIP, gdtIcons, hsNavFocusRingNoExtraInfo, rectangleP, hsNavFocusRingStyleRound, false);
				break;
		}
	}
}

//
static void drawPhoneUIBackground(SettingType *settingP)
{
	//ʾɫ
	WinSetBackColorRGB(&settingP->formFillColor, NULL);
	WinEraseWindow();

	if (settingP->shouldShowBackgd && globeP->bmpBackgdP)	//ʾͼ
	{
		//汳ͼ
		WinDrawBitmap((BitmapType *)globeP->bmpBackgdP, 0, 0);
	}
}

static void drawPhoneUISignalBattery(SettingType *settingP)
{
	UInt16				phoneRssi;
	UInt8				i;
	RGBColorType 		blackColor = {0, 0, 0, 0};
//	PhnDataSessionStateType dataSessionState;
	UInt8				phoneBattery;
	Boolean				batteryPlugin = false;
	UInt16 				barLevel;

	//ָֻ֧߷ֱĻ
	if (!globeP->isHighDensitySupported)
		return;

	WinSetCoordinateSystem(kCoordinatesDouble);

	if (globeP->isPhoneOn)
	{
		phoneRssi = getSignalLevel();

		WinSetForeColorRGB(&blackColor, NULL);

		WinPaintLines(SIGNAL_ONLY_LENGTH, signalOnly);

		for (i = 1; i <= phoneRssi; i++)
		{
			WinPaintLines(3, signalLevel[i-1]);
		}

/*	
		if (PhnLibNetworkAvailable(globeP->hsPhoneLibRef))
			WinPaintLines(SIGNAL_DATA_LENGTH, signalData);

		if (PhnLibGetDataSessionInfo(globeP->hsPhoneLibRef, &dataSessionState) != errNone)
		{
			if
			(
				dataSessionState.sessionStatus == phnPacketDataSessionActive
				||
				dataSessionState.sessionStatus == phnPacketDataSessionDormant
			)
			{
				RGBColorType greenColor = {0, 0, 255, 0};

				WinSetForeColorRGB(&greenColor, NULL);
				WinPaintLines(SIGNAL_CONNECTION_LENGTH, signalConnection);

				WinSetForeColorRGB(&blackColor, NULL);
			}
			else if
			(
				dataSessionState.availableService != phnDataServiceNone
				&&
				isValidDataService(dataSessionState.availableService)
			)
			{
				WinPaintLines(SIGNAL_DATA_LENGTH, signalData);
			}
		}
*/
	}

	WinSetForeColorRGB(&blackColor, NULL);
	
	//
	WinPaintLines(BATTERY_SHAPE_LENGTH, batteryShape);
	phoneBattery = getBatteryPercentage(&batteryPlugin);
	barLevel = 22 - phoneBattery * (22 - 4) / 100;
	
	//ɫ
	if (phoneBattery <= 20)
	{
		RGBColorType redColor = {0, 204, 0, 51};
		UInt8 i;

		WinSetForeColorRGB(&redColor, NULL);
		for (i = 0; i < 7; i++)
		{
			WinPaintLine(306 + i, 22, 306 + i, barLevel);
		}
	}	
	else	//ɫʾ
	{
		RGBColorType 	lineColor;
		
		lineColor.index = 0;

		lineColor.r = 255;
		lineColor.g = 255;
		lineColor.b = 255;
		
		WinSetForeColorRGB(&lineColor, NULL);
		WinPaintLine(306, 4, 306, barLevel);
		WinPaintLine(312, 4, 312, barLevel);
		WinPaintLine(307, 4, 307, barLevel);
		WinPaintLine(311, 4, 311, barLevel);
		if (phoneBattery != 100)
		{
			WinPaintLine(308, 2, 308, barLevel);
			WinPaintLine(309, 2, 309, barLevel);
			WinPaintLine(310, 2, 310, barLevel);
		}
		
		lineColor.r = 0;
		lineColor.g = 0;
		lineColor.b = 100;
		WinSetForeColorRGB(&lineColor, NULL);
		WinPaintLine(306, 22, 306, barLevel);
		WinPaintLine(312, 22, 312, barLevel);

		lineColor.r = 0;
		lineColor.g = 48;
		lineColor.b = 255;
		WinSetForeColorRGB(&lineColor, NULL);
		WinPaintLine(307, 22, 307, barLevel);
		WinPaintLine(311, 22, 311, barLevel);

		lineColor.r = 0;
		lineColor.g = 101;
		lineColor.b = 255;
		WinSetForeColorRGB(&lineColor, NULL);
		if (phoneBattery != 100)
		{
			WinPaintLine(308, 22, 308, barLevel);
			WinPaintLine(310, 22, 310, barLevel);
		}
		else
		{
			WinPaintLine(308, 22, 308, 2);
			WinPaintLine(310, 22, 310, 2);
		}
		lineColor.r = 100;
		lineColor.g = 207;
		lineColor.b = 255;
		WinSetForeColorRGB(&lineColor, NULL);
		if (phoneBattery != 100)
			WinPaintLine(309, 22, 309, barLevel);
		else
			WinPaintLine(309, 22, 309, 2);
	}

	//־
	if (batteryPlugin)
	{
		if (phoneBattery != 100 && globeP->bmpPowerStatusP[0] != NULL)
			WinPaintBitmap(globeP->bmpPowerStatusP[0], 299, 6);
		else if (globeP->bmpPowerStatusP[1] != NULL)
			WinPaintBitmap(globeP->bmpPowerStatusP[1], 299, 6);
	}

	WinSetCoordinateSystem(kCoordinatesStandard);
}

//״̬
static void drawPhoneUIStatusBar(SettingType *settingP)
{
	if (! globeP->isMenuActived)
	{
		UInt8		phoneSoundVolume;
		Boolean		isVibrateON;
		UInt16		stringLength;

		//״̬
//		WinSetBackColorRGB(&settingP->topStatusFillColor, NULL);
//		WinEraseRectangle(&globeP->statusBarRectangle, 0);

		//־
		WinSetTextColorRGB(&settingP->textColor, NULL);		
		FntSetFont(boldFont);
		paintCharsWithOutline(globeP->operatorNameP, StrLen(globeP->operatorNameP), 1, 1, &settingP->textOutlineColor);
		FntSetFont(stdFont);

		//Gadget
//		PmSysGadgetStatusGadgetsUpdate(globeP->pmGadgetLibRef);

		//״̬
		phoneSoundVolume = getToneStatus(&isVibrateON);

		if (phoneSoundVolume == 0)	//
		{
			WinPaintBitmap(globeP->bmpSoundStatusP[1], 105, 1);
		}
		else if (phoneSoundVolume <= 7)	//
		{
			WinPaintBitmap(globeP->bmpSoundStatusP[0], 105, 1);
			WinPaintBitmap(globeP->bmpSoundStatusP[phoneSoundVolume + 1], 112, 7);
		}
		if (isVibrateON)	//𶯿
		{
			WinPaintBitmap(globeP->bmpSoundStatusP[9], 112, 1);
		}

		//ʾ
		for (stringLength = 0; stringLength < globeP->pluginListCount; stringLength ++)
		{
			if (globeP->pluginListP[stringLength].type == ZL_PLUGIN_LIST_DB_TYPE)
			{
				ZLPluginFuncPtr aZLPluginFuncPtrs[ZLPluginNumFuncPtrs];
				UInt32 			result;

				SysAppLaunch
				(
					globeP->pluginListP[stringLength].cardNo,
					globeP->pluginListP[stringLength].dbId,
					0,
					sysAppLaunchCmdZLPluginLoad,
					aZLPluginFuncPtrs,
					&result
				);
				if (result == errNone)
				{
					ZLPluginLaunchCmdBlockTypeEx blockTypeEx;

					blockTypeEx.creatorId = globeP->pluginListP[stringLength].creatorId;
					blockTypeEx.cmdBlockType.pZLFuncPtrs = globeP->ZLFuncPtr;
					blockTypeEx.cmdBlockType.dbP = DmOpenDatabase(globeP->pluginListP[stringLength].cardNo, globeP->pluginListP[stringLength].dbId, dmModeReadOnly);
					if (blockTypeEx.cmdBlockType.dbP != NULL)
					{
						//״̬
						if (globeP->pluginListP[stringLength].creatorId == 'ZLST')
						{
							blockTypeEx.cmdBlockType.position = ZLPluginPosButton;
							blockTypeEx.cmdBlockType.buttonType = ZLPluginButtonTop;

							blockTypeEx.cmdBlockType.lefty = false;
							blockTypeEx.cmdBlockType.pluginMgrVerNum = ZLPluginMgrVerNum;
							blockTypeEx.cmdBlockType.ZLPluginPanelWidth = globeP->pluginListP[stringLength].pluginRectangle.extent.x;
							blockTypeEx.cmdBlockType.ZLPluginPanelHeight = globeP->pluginListP[stringLength].pluginRectangle.extent.y;
							blockTypeEx.cmdBlockType.bounds = globeP->pluginListP[stringLength].pluginRectangle;
							//ҪUIͬĴС
							blockTypeEx.cmdBlockType.clipBounds.topLeft.x = 118;
							blockTypeEx.cmdBlockType.clipBounds.topLeft.y = 1;
							blockTypeEx.cmdBlockType.clipBounds.extent.x = 10;
							blockTypeEx.cmdBlockType.clipBounds.extent.y = 10;

							result = aZLPluginFuncPtrs[ZLPluginFuncPtrOpen]((ZLPluginLaunchCmdBlockType *)&blockTypeEx);
							if (result == errNone)
							{
								aZLPluginFuncPtrs[ZLPluginFuncPtrRefresh]((ZLPluginLaunchCmdBlockType *)&blockTypeEx);
								aZLPluginFuncPtrs[ZLPluginFuncPtrClose]((ZLPluginLaunchCmdBlockType *)&blockTypeEx);
							}
						}

						DmCloseDatabase(blockTypeEx.cmdBlockType.dbP);
					}
				}
			}
		}

		//źǿȺ͵ʾ
		drawPhoneUISignalBattery(settingP);
	}
}

//
static void drawPhoneUICalendar(DateTimeType *nowP, SettingType *settingP)
{
	RGBColorType	colorGray		= {0, 204, 204, 204};
	RGBColorType	colorRed		= {0, 255, 0, 0};
	RGBColorType	colorBlue		= {0, 0, 0, 255};
	const Char		*weekTitle[7]	= {STRING_WEEK_TITLE};
	UInt8			daysInMonth		= DaysInMonth(nowP->month, nowP->year);
	UInt8			day, i;
	Coord			x, y;
	UInt16			stringLength;
	Char			dayBufferP[maxStrIToALen];

	WinSetTextColorRGB(&settingP->textColor, NULL);

	//
	for (i = 0; i < 7; i ++)
	{
		x = 3 + 22 * i;
		x += 11 - FntCharsWidth(weekTitle[i], WEEK_TITLE_LENGTH) / 2;
		if (i == 5)
		{
			WinSetTextColorRGB(&colorBlue, NULL);
			paintCharsWithOutline(weekTitle[i], WEEK_TITLE_LENGTH, x, 29, &settingP->textOutlineColor);
			WinSetTextColorRGB(&settingP->textColor, NULL);
		}
		else if (i == 6)
		{
			WinSetTextColorRGB(&colorRed, NULL);
			paintCharsWithOutline(weekTitle[i], WEEK_TITLE_LENGTH, x, 29, &settingP->textOutlineColor);
			WinSetTextColorRGB(&settingP->textColor, NULL);
		}
		else
		{
			paintCharsWithOutline(weekTitle[i], WEEK_TITLE_LENGTH, x, 29, &settingP->textOutlineColor);
		}
	}

	//
	i = DayOfWeek(nowP->month, 1, nowP->year);
	if (i == 0)
		i = 7;
	y = 40;
	for (day = 1; day <= daysInMonth; day ++)
	{
		x = 22 * i - 19;	// x = 3 + 22 * (i - 1);
		StrIToA(dayBufferP, day);
		stringLength = StrLen(dayBufferP);

		if (i == 6)
		{
			WinSetTextColorRGB(&colorBlue, NULL);
		}
		else if (i == 7)
		{
			WinSetTextColorRGB(&colorRed, NULL);
		}
		if (day == nowP->day)
		{
			FntSetFont(boldFont);
		}

		x += 11 - FntCharsWidth(dayBufferP, stringLength) / 2;
		paintCharsWithOutline(dayBufferP, stringLength, x, y, &settingP->textOutlineColor);

		if (day == nowP->day)
		{
			FntSetFont(stdFont);
		}
		if (i == 6 || i == 7)
		{
			WinSetTextColorRGB(&settingP->textColor, NULL);
		}

		i++;
		if (i > 7)
		{
			i = 1;
			y += 11;
		}
	}
}

//
static void drawPhoneUIContent(SettingType *settingP)
{
	UInt16			stringLength;
	UInt32			i;
	DateTimeType	now;

	//ȡǰʱ
	TimSecondsToDateTime(TimGetSeconds(), &now);

	if (globeP->isInPhoneView)
	{
		if (settingP->shouldShowDateTime)	//ʾʱ
		{
			//ȡʱ
			DateTemplateToAscii(STRING_DATE_FORMAT, now.month, now.day, now.year, globeP->dateStringP, 64);
			if (StrStr(STRING_DATE_FORMAT, ""))		//ģʽ
			{
				switch (now.weekDay)
				{
					case 0:
						StrCat(globeP->dateStringP, "");
						break;
					case 1:
						StrCat(globeP->dateStringP, "һ");
						break;
					case 2:
						StrCat(globeP->dateStringP, "");
						break;
					case 3:
						StrCat(globeP->dateStringP, "");
						break;
					case 4:
						StrCat(globeP->dateStringP, "");
						break;
					case 5:
						StrCat(globeP->dateStringP, "");
						break;
					case 6:
						StrCat(globeP->dateStringP, "");
						break;
				}
			}
			TimeToAscii(now.hour, now.minute, tfColon24h, globeP->timeStringP);
			stringLength = StrLen(globeP->timeStringP);
			FntSetFont(boldFont);

			WinSetForeColorRGB(&settingP->formFrameColor, NULL);
			if (settingP->shouldAddMask)	//ֲ
			{
				WinPaintRectangle(&globeP->datetimeBarRectangle, 0);

				WinSetTextColorRGB(&settingP->textOnMaskColor, NULL);
				WinPaintChars(globeP->dateStringP, StrLen(globeP->dateStringP), 3, 15);
				WinPaintChars(globeP->timeStringP, stringLength, 158 - FntCharsWidth(globeP->timeStringP, stringLength), 15);
			}
			else
			{
				if (settingP->shouldAddLine)
				{
					WinPaintLine(0, 13, 159, 13);
					WinPaintLine(0, 27, 159, 27);
				}

				WinSetTextColorRGB(&settingP->textColor, NULL);
				paintCharsWithOutline(globeP->dateStringP, StrLen(globeP->dateStringP), 3, 15, &settingP->textOutlineColor);
				paintCharsWithOutline(globeP->timeStringP, stringLength, 158 - FntCharsWidth(globeP->timeStringP, stringLength), 15, &settingP->textOutlineColor);
			}

			FntSetFont(stdFont);

			if (settingP->shouldShowCalendar)	//ʾ
			{
				drawPhoneUICalendar(&now, settingP);
			}
		}
		else if (settingP->shouldAddLine)
		{
			WinSetForeColorRGB(&settingP->formFrameColor, NULL);
			WinPaintLine(0, 13, 159, 13);
		}

		//ʾ
		for (stringLength = 0; stringLength < globeP->pluginListCount; stringLength ++)
		{
			//NeoʽĲ
			if (globeP->pluginListP[stringLength].creatorId == APPLICATION_CREATOR)
			{
				globeP->pluginParameter.actionType = ACTION_TYPE_DRAW;
				globeP->pluginParameter.pluginRectangle = globeP->pluginListP[stringLength].pluginRectangle;
				globeP->pluginParameter.pluginDataP = globeP->pluginListP[stringLength].pluginDataP;
				SysAppLaunch(globeP->pluginListP[stringLength].cardNo, globeP->pluginListP[stringLength].dbId, 0, LAUNCH_CODE_PLUGIN_ACTION, &globeP->pluginParameter, &i);
				globeP->pluginListP[stringLength].pluginDataP = globeP->pluginParameter.pluginDataP;
				globeP->pluginListP[stringLength].pluginRectangle = globeP->pluginParameter.pluginRectangle;
			}
			else if (globeP->pluginListP[stringLength].type == ZL_PLUGIN_LIST_DB_TYPE)
			{
				ZLPluginFuncPtr aZLPluginFuncPtrs[ZLPluginNumFuncPtrs];
				SysAppLaunch(globeP->pluginListP[stringLength].cardNo, globeP->pluginListP[stringLength].dbId, 0, sysAppLaunchCmdZLPluginLoad, aZLPluginFuncPtrs, &i);
				if (i == errNone)
				{
					ZLPluginLaunchCmdBlockTypeEx blockTypeEx;

					blockTypeEx.creatorId = globeP->pluginListP[stringLength].creatorId;
					blockTypeEx.cmdBlockType.pZLFuncPtrs = globeP->ZLFuncPtr;
					blockTypeEx.cmdBlockType.dbP = DmOpenDatabase(globeP->pluginListP[stringLength].cardNo, globeP->pluginListP[stringLength].dbId, dmModeReadOnly);
					if (blockTypeEx.cmdBlockType.dbP != NULL)
					{
						//״̬
						if (globeP->pluginListP[stringLength].creatorId != 'ZLST')
						{
							blockTypeEx.cmdBlockType.position = ZLPluginPosPanel;
							blockTypeEx.cmdBlockType.panelType = ZLPluginPanelTop;
							if (!settingP->shouldShowCalendar)
							{
								blockTypeEx.cmdBlockType.lefty = false;
								blockTypeEx.cmdBlockType.pluginMgrVerNum = ZLPluginMgrVerNum;
								blockTypeEx.cmdBlockType.ZLPluginPanelWidth = globeP->pluginListP[stringLength].pluginRectangle.extent.x;
								blockTypeEx.cmdBlockType.ZLPluginPanelHeight = globeP->pluginListP[stringLength].pluginRectangle.extent.y;
								blockTypeEx.cmdBlockType.bounds = globeP->pluginListP[stringLength].pluginRectangle;
								//UIͬĴС
								blockTypeEx.cmdBlockType.clipBounds.topLeft.x = 0;
								blockTypeEx.cmdBlockType.clipBounds.topLeft.y = 0;
								blockTypeEx.cmdBlockType.clipBounds.extent.x = 160;
								blockTypeEx.cmdBlockType.clipBounds.extent.y = 160;

								i = aZLPluginFuncPtrs[ZLPluginFuncPtrOpen]((ZLPluginLaunchCmdBlockType *)&blockTypeEx);
								if (i == errNone)
								{
									aZLPluginFuncPtrs[ZLPluginFuncPtrRefresh]((ZLPluginLaunchCmdBlockType *)&blockTypeEx);
									aZLPluginFuncPtrs[ZLPluginFuncPtrClose]((ZLPluginLaunchCmdBlockType *)&blockTypeEx);
								}
							}	
						}
						DmCloseDatabase(blockTypeEx.cmdBlockType.dbP);
					}
				}
			}
		}
	}
	else
	{
		//͸
		WinSetForeColorRGB(&settingP->formFillColor, NULL);
		WinSetPatternType(settingP->maskPattern);
		WinPaintRectangle(&globeP->launchViewRectangle, 0);
		WinSetPatternType(blackPattern);
		WinSetForeColorRGB(&settingP->formFrameColor, NULL);

		//ȡʱ
		TimeToAscii(now.hour, now.minute, tfColon24h, globeP->timeStringP);
		FntSetFont(boldFont);

		//WinSetForeColorRGB(&settingP->formFrameColor, NULL);
		//WinPaintRectangle(&globeP->datetimeBarRectangle, 0);

		//WinSetTextColorRGB(&settingP->textOnMaskColor, NULL);
		//stringLength = StrLen(globeP->timeStringP);
		//WinPaintChars(globeP->timeStringP, stringLength, 158 - FntCharsWidth(globeP->timeStringP, stringLength), 15);

		stringLength = StrLen(globeP->timeStringP);
		if (settingP->shouldAddMask)	//ֲ
		{
			WinPaintRectangle(&globeP->datetimeBarRectangle, 0);

			WinSetTextColorRGB(&settingP->textOnMaskColor, NULL);
			WinPaintChars(globeP->timeStringP, stringLength, 158 - FntCharsWidth(globeP->timeStringP, stringLength), 15);
		}
		else
		{
			if (settingP->shouldAddLine)
			{
				WinPaintLine(0, 13, 159, 13);
				WinPaintLine(0, 27, 159, 27);
			}

			WinSetTextColorRGB(&settingP->textColor, NULL);
			paintCharsWithOutline(globeP->timeStringP, stringLength, 158 - FntCharsWidth(globeP->timeStringP, stringLength), 15, &settingP->textOutlineColor);
		}

		FntSetFont(stdFont);
	}
	
	globeP->justNow = now;
}

//
static void drawPhoneUILaunchBar(SettingType *settingP)
{
	RectangleType		highlightRectangle;
	ProgramItemInfoType	programInfo;
	UInt16				nameLength;
	Coord				x;
	Coord				y;
	Coord				width;
	Coord				height;
	UInt16				i;
	UInt16				row					= 0;
	UInt16				col					= 0;
	UInt16				currentProgram		= globeP->beginProgramInPage + globeP->currentCol + 4 * globeP->currentRow;

	//ԭ
	WinSetForeColorRGB(&settingP->formFrameColor, NULL);
	WinSetBackColorRGB(&settingP->formFillColor, NULL);
	WinSetTextColorRGB(&settingP->textColor, NULL);

	if (globeP->isInPhoneView)	//绰ģʽĻײ
	{
		//ĴСλ
		globeP->positionRectangle[0].topLeft.y = 121; globeP->positionRectangle[0].extent.x = 36; globeP->positionRectangle[0].extent.y = 36;
		globeP->positionRectangle[1].topLeft.y = 121; globeP->positionRectangle[1].extent.x = 36; globeP->positionRectangle[1].extent.y = 36;
		globeP->positionRectangle[2].topLeft.y = 121; globeP->positionRectangle[2].extent.x = 36; globeP->positionRectangle[2].extent.y = 36;
		globeP->positionRectangle[3].topLeft.y = 121; globeP->positionRectangle[3].extent.x = 36; globeP->positionRectangle[3].extent.y = 36;
	}
	else	//ģʽ
	{
		if (settingP->shouldAddLine)
		{
			WinPaintLine(0, 146, 159, 146);
		}

		if (settingP->shouldShowSearch)
		{
			if (globeP->searchKeywordNum != 0)
			{
				StrCopy(globeP->cacheP, STRING_FIND);
				StrCat(globeP->cacheP, globeP->searchKeyword); 
				paintCharsWithOutline(globeP->cacheP, StrLen(globeP->cacheP), 110, 148, &settingP->textOutlineColor);
			}
		}

		//״̬ͷҳͷ
		FntSetFont(symbol7Font);
		x =  80 - FntCharWidth('\1') / 2;
		if (globeP->beginProgramInPage == 0)
		{
			if (globeP->currentRow == 0)
			{
				WinSetTextColorRGB(&settingP->formFrameColor, NULL);
				paintCharWithOutline('\1', x, 146, &settingP->textOutlineColor);
				WinSetTextColorRGB(&settingP->textColor, NULL);
			}
			else
			{
				paintCharWithOutline('\3', x, 146, &settingP->textOutlineColor);
			}
		}
		else
		{
			paintCharWithOutline('\1', x, 146, &settingP->textOutlineColor);
		}
		if (globeP->endProgramInPage == globeP->programListCount - 1 || globeP->programListCount == 0)
		{
			paintCharWithOutline('\4', x, 153, &settingP->textOutlineColor);
		}
		else
		{
			paintCharWithOutline('\2', x, 153, &settingP->textOutlineColor);
		}
		FntSetFont(stdFont);

		//ĴСλ
		globeP->positionRectangle[0].topLeft.y = 31; globeP->positionRectangle[0].extent.x = 36; globeP->positionRectangle[0].extent.y = 36;
		globeP->positionRectangle[1].topLeft.y = 31; globeP->positionRectangle[1].extent.x = 36; globeP->positionRectangle[1].extent.y = 36;
		globeP->positionRectangle[2].topLeft.y = 31; globeP->positionRectangle[2].extent.x = 36; globeP->positionRectangle[2].extent.y = 36;
		globeP->positionRectangle[3].topLeft.y = 31; globeP->positionRectangle[3].extent.x = 36; globeP->positionRectangle[3].extent.y = 36;
	}
	globeP->positionRectangle[0].topLeft.x = 3;
	globeP->positionRectangle[1].topLeft.x = 42;
	globeP->positionRectangle[2].topLeft.x = 82;
	globeP->positionRectangle[3].topLeft.x = 121;

	//ͼ
	WinSetForeColorRGB(&settingP->textColor, NULL);
	WinSetTextColorRGB(&settingP->textColor, NULL);
	if (globeP->programListCount > 0)
	{
		//
		if (globeP->focusedObjectId == gdtIcons && settingP->focusRingStyle > FOCUS_RING_STYLE_ANGLE)
		{
			highlightRectangle = globeP->positionRectangle[globeP->currentCol];
			highlightRectangle.topLeft.y += 38 * globeP->currentRow;
			drawIconFocusedRing(&highlightRectangle, settingP->focusRingStyle, settingP);
		}

		for (i = globeP->beginProgramInPage; i <= globeP->endProgramInPage; i ++)
		{
			//ȡλ
			x = globeP->positionRectangle[col].topLeft.x;
			y = globeP->positionRectangle[col].topLeft.y + 38 * row;

			//ȡƺͼ
			getProgramItemInfo(settingP->deviceModel, &globeP->programListP[i], &programInfo);

			//
			nameLength = StrLen(programInfo.nameP);
			width = FntCharsWidth(programInfo.nameP, nameLength);
			if (width <= globeP->positionRectangle[0].extent.x)
			{
				paintCharsWithOutline(programInfo.nameP, nameLength, x + (globeP->positionRectangle[0].extent.x - width) / 2, y + 24, &settingP->textOutlineColor);
			}
			else
			{
				winPaintTruncChars(programInfo.nameP, nameLength, x, y + 24, globeP->positionRectangle[0].extent.x, true, &settingP->textOutlineColor);
			}

			//
			if (i == currentProgram && (! globeP->isInPhoneView))
			{
				FntSetFont(boldFont);
				if (settingP->shouldAddMask)
				{
					WinSetTextColorRGB(&settingP->textOnMaskColor, NULL);
					if (width <= 100)
					{
						WinPaintChars(programInfo.nameP, nameLength, 2, 15);
					}
					else
					{
						winPaintTruncChars(programInfo.nameP, nameLength, 2, 15, 100, false, NULL);
					}
					WinSetTextColorRGB(&settingP->textColor, NULL);
				}
				else
				{
					if (width <= 100)
					{
						paintCharsWithOutline(programInfo.nameP, nameLength, 2, 15, &settingP->textOutlineColor);
					}
					else
					{
						winPaintTruncChars(programInfo.nameP, nameLength, 2, 15, 100, true, &settingP->textOutlineColor);
					}
				}
				FntSetFont(stdFont);
			}

			//ͼ
			if (programInfo.bmpIconP != NULL)
			{
				BmpGetDimensions(programInfo.bmpIconP, &width, &height, NULL);
				if (width > 32)
				{
					width /= 2;
				}
				if (height > 22)
				{
					height = 22;
				}
				WinPaintBitmap(programInfo.bmpIconP, x + 18 - width / 2, y + 13 - height / 2);
			}
			
			//̱
			if (globeP->programListP[i].VFSNameP[0] != '\0')
			{
				paintCharWithOutline('\32', x + 36 - FntCharWidth('\32'), y, &settingP->textOutlineColor);
			}

			//ݼ
			if (globeP->programListP[i].shortcut != chrNull)
			{
				paintCharWithOutline(globeP->programListP[i].shortcut - 32, x + 36 - FntCharWidth(globeP->programListP[i].shortcut), y, &settingP->textOutlineColor);
			}

			//ͼ껺
			if (programInfo.bmpIconP != NULL)
			{
				MemPtrFree(programInfo.bmpIconP);
				programInfo.bmpIconP = NULL;
			}

			col ++;
			if (col > 3)
			{
				row ++;
				col = 0;
			}
		}

		//
		if (globeP->focusedObjectId == gdtIcons && settingP->focusRingStyle < FOCUS_RING_STYLE_ROUND_FRAME)
		{
			highlightRectangle = globeP->positionRectangle[globeP->currentCol];
			highlightRectangle.topLeft.y += 38 * globeP->currentRow;
			drawIconFocusedRing(&highlightRectangle, settingP->focusRingStyle, settingP);
		}

		//¼ͷ
//		if (globeP->isInPhoneView)
//		{
//			FntSetFont(symbol7Font);
//			paintCharsWithOutline("\2", 1, 80 - FntCharWidth('\2') / 2, 153, &settingP->textOutlineColor);
//			FntSetFont(stdFont);
//		}
	}
}

//绰
static void drawPhoneUI(UInt16 code, SettingType *settingP)
{
	switch (code)
	{
		case FORM_UPDATE_REFRESH:	//ȫ
			getOperatorName(settingP);
			if (globeP->bmpBackgdP != NULL)
			{
				BmpDelete((BitmapType *)globeP->bmpBackgdP);
			}
			globeP->bmpBackgdP = getBackgdBitmap(settingP);
			//
		case frmRedrawUpdateCode:
		case FORM_UPDATE_STATUS_BAR_ONLY:
		case FORM_UPDATE_LAUNCH_BAR_ONLY:
		case FORM_UPDATE_CONTENT:
			//Ƴ
			HsNavRemoveFocusRing(globeP->frmPhoneUIP);

			//ڻлͼ
			WinSetDrawWindow(globeP->offscreenWindowH);

			//Ʊ
			drawPhoneUIBackground(settingP);

			//״̬
			drawPhoneUIStatusBar(settingP);

			//
			drawPhoneUIContent(settingP);

			//
			if
			(
				!globeP->isInPhoneView
				||
				settingP->shouldShowTopIcons
			)
				drawPhoneUILaunchBar(settingP);

			//Ļ
			WinSetDrawWindow((WinHandle)globeP->frmPhoneUIP);
			WinCopyRectangle(globeP->offscreenWindowH, (WinHandle)globeP->frmPhoneUIP, &globeP->backgdRectangle, 0, 0, winPaint);
			break;
	}

	WinSetBackColorRGB(&settingP->topStatusFillColor, NULL);

	if (globeP->focusedObjectId != gdtIcons)
	{
		HsNavDrawFocusRing(globeP->frmPhoneUIP, globeP->focusedObjectId, hsNavFocusRingNoExtraInfo, getFocusedObjectBounds(globeP->focusedObjectId), hsNavFocusRingStyleSquare, true);
	}
}

//ʾʾ
static void drawKeyHoldTip(SettingType *settingP)
{
	RectangleType	r				= {0, 139, 160, 20};
	UInt16			stringLength	= StrLen(STRING_KEY_HOLD_TIP);

	WinSetBackColorRGB(&settingP->formFrameColor, NULL);
	WinSetTextColorRGB(&settingP->formFillColor, NULL);

	WinEraseRectangle(&r, 2);

	FntSetFont(boldFont);
	WinPaintChars(STRING_KEY_HOLD_TIP, stringLength, 80 - FntCharsWidth(STRING_KEY_HOLD_TIP, stringLength) / 2, 145);
	FntSetFont(stdFont);

	WinSetBackColorRGB(&settingP->formFillColor, NULL);
	WinSetTextColorRGB(&settingP->textColor, NULL);
}

#if 0
#pragma mark == EVENT HANDLER ==
#endif

//лʾģʽ
static void viewSwitch(UInt8 viewMode, SettingType *settingP)
{
	switch (viewMode)
	{
		case VIEW_MODE_PHONE:	//ת绰
			globeP->isInPhoneView = true;
			getProgramListInCategory(ALL_CATEGORIES);
			globeP->currentRow = 0;
			globeP->currentCol = 0;
			globeP->beginProgramInPage = 0;
			globeP->endProgramInPage = 3;
			globeP->searchKeywordNum = 0;
			globeP->focusedObjectId = gdtIcons;
			HsNavObjectTakeFocus(globeP->frmPhoneUIP, gdtIcons);
			break;
		case VIEW_MODE_PROGRAM:	//ת
			globeP->focusedObjectId = gdtIcons;
			globeP->isInPhoneView = false;
			globeP->searchKeywordNum = 0;
			getProgramListInCategory(settingP->lastCategory);
			globeP->currentRow = 0;
			globeP->currentCol = 0;
			globeP->beginProgramInPage = 0;
			globeP->endProgramInPage = 11;
			if (globeP->programListCount == 0)
			{
				globeP->endProgramInPage = 0;
				globeP->focusedObjectId = ptriCategory;
			}
			else if (globeP->endProgramInPage > globeP->programListCount - 1)
			{
				globeP->endProgramInPage = globeP->programListCount - 1;
			}
			break;
	}

	globeP->isInSearchMode = false;
}

//ѡť
static Boolean isPenDownOnIcons(Coord x, Coord y, UInt16 *selectedProgram, UInt8 *row, UInt8 *column)
{
	globeP->positionRectangle[0].topLeft.x = 3;
	globeP->positionRectangle[1].topLeft.x = 42;
	globeP->positionRectangle[2].topLeft.x = 82;
	globeP->positionRectangle[3].topLeft.x = 121;
	if (globeP->isInPhoneView)
	{
		globeP->positionRectangle[0].topLeft.y = 121; globeP->positionRectangle[0].extent.x = 36; globeP->positionRectangle[0].extent.y = 36;
		globeP->positionRectangle[1].topLeft.y = 121; globeP->positionRectangle[1].extent.x = 36; globeP->positionRectangle[1].extent.y = 36;
		globeP->positionRectangle[2].topLeft.y = 121; globeP->positionRectangle[2].extent.x = 36; globeP->positionRectangle[2].extent.y = 36;
		globeP->positionRectangle[3].topLeft.y = 121; globeP->positionRectangle[3].extent.x = 36; globeP->positionRectangle[3].extent.y = 36;

		(*row) = 0;
		for ((*column) = 0; (*column) < 4; (*column) ++)
		{
			if (RctPtInRectangle(x, y, &globeP->positionRectangle[(*column)]))
			{
				(*selectedProgram) = (*column);
				return true;
			}
		}
	}
	else
	{
		globeP->positionRectangle[0].topLeft.y = 31; globeP->positionRectangle[0].extent.x = 36; globeP->positionRectangle[0].extent.y = 36;
		globeP->positionRectangle[1].topLeft.y = 31; globeP->positionRectangle[1].extent.x = 36; globeP->positionRectangle[1].extent.y = 36;
		globeP->positionRectangle[2].topLeft.y = 31; globeP->positionRectangle[2].extent.x = 36; globeP->positionRectangle[2].extent.y = 36;
		globeP->positionRectangle[3].topLeft.y = 31; globeP->positionRectangle[3].extent.x = 36; globeP->positionRectangle[3].extent.y = 36;

		for ((*column) = 0; (*column) < 4; (*column) ++)
		{
			for ((*row) = 0; (*row) < 3; (*row) ++)
			{
				if (RctPtInRectangle(x, y, &globeP->positionRectangle[(*column)]))
				{
					(*selectedProgram) = (*column) + (globeP->beginProgramInPage + ((*row) * 4));
					if ((*selectedProgram) <= globeP->endProgramInPage)
					{
						return true;
					}
					else
					{
						break;
					}
				}
				globeP->positionRectangle[(*column)].topLeft.y += 38;
			}
		}
	}
	return false;
}

//Ƿܼ
static Boolean shouldPenDownActivePlugin(EventType *eventP)
{
	UInt32		i;

	for (i = 0; i < globeP->pluginListCount; i ++)
	{
		if (globeP->pluginListP[i].creatorId == APPLICATION_CREATOR)
		{
			if (RctPtInRectangle(eventP->screenX, eventP->screenY, &globeP->pluginListP[i].pluginRectangle))
			{
				globeP->pluginParameter.actionType = ACTION_TYPE_TAP;
				globeP->pluginParameter.screenX = eventP->screenX;
				globeP->pluginParameter.screenY = eventP->screenY;
				globeP->pluginParameter.pluginRectangle = globeP->pluginListP[i].pluginRectangle;
				globeP->pluginParameter.pluginDataP = globeP->pluginListP[i].pluginDataP;
				SysAppLaunch(globeP->pluginListP[i].cardNo, globeP->pluginListP[i].dbId, 0, LAUNCH_CODE_PLUGIN_ACTION, &globeP->pluginParameter, &i);
				globeP->pluginListP[i].pluginDataP = globeP->pluginParameter.pluginDataP;
				return true;
			}
		}
/*		
		else if (globeP->pluginListP[i].type == ZL_PLUGIN_LIST_DB_TYPE && !settingP->shouldShowCalendar)
		{
			if (RctPtInRectangle(eventP->screenX, eventP->screenY, &globeP->pluginListP[i].pluginRectangle))
			{
				ZLPluginFuncPtr aZLPluginFuncPtrs[ZLPluginNumFuncPtrs];
				UInt32 			result;

				SysAppLaunch
				(
					globeP->pluginListP[i].cardNo,
					globeP->pluginListP[i].dbId,
					0,
					sysAppLaunchCmdZLPluginLoad,
					aZLPluginFuncPtrs,
					&result
				);
				if (result == errNone)
				{
					ZLPluginLaunchCmdBlockTypeEx blockTypeEx;
					ZLPluginLaunchCmdEventType eventType;

					eventType.handled = false;
					eventType.eventP = eventP;
					blockTypeEx.cmdBlockType.dataP = (void *)&eventType;
					blockTypeEx.creatorId = globeP->pluginListP[i].creatorId;
					blockTypeEx.cmdBlockType.pZLFuncPtrs = globeP->ZLFuncPtr;
					blockTypeEx.cmdBlockType.dbP = DmOpenDatabase(globeP->pluginListP[i].cardNo, globeP->pluginListP[i].dbId, dmModeReadOnly);
					if (blockTypeEx.cmdBlockType.dbP != NULL)
					{
						//״̬
						if (globeP->pluginListP[i].creatorId != 'ZLST')
						{
							blockTypeEx.cmdBlockType.position = ZLPluginPosTab;
							blockTypeEx.cmdBlockType.panelType = ZLPluginPanelTop;
							blockTypeEx.cmdBlockType.lefty = false;
							blockTypeEx.cmdBlockType.pluginMgrVerNum = ZLPluginMgrVerNum;
							blockTypeEx.cmdBlockType.ZLPluginPanelWidth = globeP->pluginListP[i].pluginRectangle.extent.x;
							blockTypeEx.cmdBlockType.ZLPluginPanelHeight = globeP->pluginListP[i].pluginRectangle.extent.y;
							blockTypeEx.cmdBlockType.bounds = globeP->pluginListP[i].pluginRectangle;
							//UIͬĴС
							blockTypeEx.cmdBlockType.clipBounds.topLeft.x = 0;
							blockTypeEx.cmdBlockType.clipBounds.topLeft.y = 0;
							blockTypeEx.cmdBlockType.clipBounds.extent.x = 160;
							blockTypeEx.cmdBlockType.clipBounds.extent.y = 160;

							i = aZLPluginFuncPtrs[ZLPluginFuncPtrOpen]((ZLPluginLaunchCmdBlockType *)&blockTypeEx);
							if (i == errNone)
							{
								aZLPluginFuncPtrs[ZLPluginFuncPtrHandleEvent]((ZLPluginLaunchCmdBlockType *)&blockTypeEx);
								aZLPluginFuncPtrs[ZLPluginFuncPtrClose]((ZLPluginLaunchCmdBlockType *)&blockTypeEx);
							}
						}
						DmCloseDatabase(blockTypeEx.cmdBlockType.dbP);
					}
					return eventType.handled;
				}
			}	
		}
*/		
	}

	return false;
}

//Ƿܼ
static Boolean shouldKeyActivePlugin(WChar chr)
{
	UInt32		i;

	for (i = 0; i < globeP->pluginListCount; i ++)
	{
		if (globeP->pluginListP[i].creatorId == APPLICATION_CREATOR)
		{
			if (globeP->pluginListP[i].activeKey == chr)
			{
				globeP->pluginParameter.actionType = ACTION_TYPE_KEY;
				globeP->pluginParameter.pluginRectangle = globeP->pluginListP[i].pluginRectangle;
				globeP->pluginParameter.pluginDataP = globeP->pluginListP[i].pluginDataP;
				SysAppLaunch(globeP->pluginListP[i].cardNo, globeP->pluginListP[i].dbId, 0, LAUNCH_CODE_PLUGIN_ACTION, &globeP->pluginParameter, &i);
				globeP->pluginListP[i].pluginDataP = globeP->pluginParameter.pluginDataP;

				return true;
			}
		}
	}

	return false;
}

//ɾӿĳ
static void deleteProgramLaunchFromCard(SettingType *settingP)
{
	if (settingP->lastProgramLaunchFromCard.creatorId != 0 && settingP->lastProgramLaunchFromCard.type != 0)
	{
		DmSearchStateType		searchState;

		if (DmGetNextDatabaseByTypeCreator(true, &searchState, settingP->lastProgramLaunchFromCard.type, settingP->lastProgramLaunchFromCard.creatorId, true, &settingP->lastProgramLaunchFromCard.cardNo, &settingP->lastProgramLaunchFromCard.dbId) == errNone)
		{
			DmDeleteDatabase(settingP->lastProgramLaunchFromCard.cardNo, settingP->lastProgramLaunchFromCard.dbId);
			MemSet(&settingP->lastProgramLaunchFromCard, sizeof(ProgramListType), 0);
			PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, PREFERENCE_VERSION, settingP, sizeof(SettingType), true);
		}
	}
}

//ǷҪʱ
static Boolean shouldUpdateDateTime()
{
	DateTimeType	now;

	//ȡǰʱ
	TimSecondsToDateTime(TimGetSeconds(), &now);

	if (now.day != globeP->justNow.day || now.minute != globeP->justNow.minute)
	{
		return true;
	}
	return false;
}

#if 0
#pragma mark -----------------------------------------------------
#endif

//龰ģʽ¼
static void profileEventHandler(SettingType *settingP)
{
	EventType		event;
	FormType		*frmProfileP;
	ControlType		*pbtnProfilesP[5];
	SoundPreference	soundPrefs;
	Int32			i;
	Boolean			isEventHandled;
	Boolean			shouldExit		= false;

	FrmPopupForm(frmProfile);

	do
	{
		EvtGetEvent(&event, evtWaitForever);
		isEventHandled = false;

		if (! SysHandleEvent(&event))
		{
			switch (event.eType)
			{
				case frmLoadEvent:
					if (event.data.frmLoad.formID == frmProfile)
					{
						frmProfileP = (FormType *)FrmInitForm(frmProfile);
						for (i = 0; i < 5; i ++)
						{
							pbtnProfilesP[i] = (ControlType *)GET_OBJECT_POINTER(frmProfileP, pbtnProfile1 + i);
						}
						FrmSetActiveForm(frmProfileP);
						isEventHandled = true;
					}
					break;
				case frmOpenEvent:
					if (event.data.frmOpen.formID == frmProfile)
					{
						FrmDrawForm(frmProfileP);
						for (i = 0; i < 5; i ++)
						{
							CtlSetLabel(pbtnProfilesP[i], settingP->profiles[i].nameP);
						}
						CtlSetValue(pbtnProfilesP[settingP->lastProfile], 1);
						HsNavObjectTakeFocus(frmProfileP, pbtnProfile1 + settingP->lastProfile);

						isEventHandled = true;
					}
					break;
				case ctlSelectEvent:
					settingP->lastProfile = event.data.ctlSelect.controlID - pbtnProfile1;
					shouldExit = true;
					break;
			}

			if (! isEventHandled)
			{
				FrmDispatchEvent(&event);
			}
		}
	}while (event.eType != appStopEvent && shouldExit == false);

	FrmReturnToForm(0);

	for (i = soundPrefTypePhone; i < soundPrefTypeLast; i ++)
	{
		TonesLibGetSoundPrefs(globeP->tonesLibRef, i, &soundPrefs);

		switch (i)
		{
			case soundPrefTypePhone:
				soundPrefs.soundOnVolume = settingP->profiles[settingP->lastProfile].phoneSoundPrefs.soundOnVolume;
				soundPrefs.soundOnVibrate = settingP->profiles[settingP->lastProfile].phoneSoundPrefs.soundOnVibrate;
				soundPrefs.soundOffVibrate = settingP->profiles[settingP->lastProfile].phoneSoundPrefs.soundOffVibrate;
				break;
			case soundPrefTypeEmail:
				break;
			case soundPrefTypeCalendar:
				soundPrefs.soundOnVolume = settingP->profiles[settingP->lastProfile].calendarSoundPrefs.soundOnVolume;
				soundPrefs.soundOnVibrate = settingP->profiles[settingP->lastProfile].calendarSoundPrefs.soundOnVibrate;
				soundPrefs.soundOffVibrate = settingP->profiles[settingP->lastProfile].calendarSoundPrefs.soundOffVibrate;
				break;
			case soundPrefTypeIM:
				break;
			case soundPrefTypeMMS:
				break;
			case soundPrefTypeSMS:
				soundPrefs.soundOnVolume = settingP->profiles[settingP->lastProfile].smsSoundPrefs.soundOnVolume;
				soundPrefs.soundOnVibrate = settingP->profiles[settingP->lastProfile].smsSoundPrefs.soundOnVibrate;
				soundPrefs.soundOffVibrate = settingP->profiles[settingP->lastProfile].smsSoundPrefs.soundOffVibrate;
				break;
		}

		TonesLibSetSoundPrefs(globeP->tonesLibRef, i, &soundPrefs);
	}
}

//˵¼
static Boolean categoryListPopupEventHandler(SettingType *settingP)
{
	Int16		selectedIndex	= LstPopupList(globeP->lstCategoryP);

	if (selectedIndex == noListSelection)
	{
		return false;
	}
	else
	{
		settingP->lastCategory = getCategoryId(selectedIndex, 0);
	}

	return true;
}

//ؼѡ¼
static Boolean controlSelectEventHandler(UInt16 objectId, SettingType *settingP)
{
	Boolean			isEventHandled	= false;

	switch (objectId)
	{
		case gdtCarrier:
			EvtEnqueueKey(vchrMenu, 0, 8);
			isEventHandled = true;
			break;
		case gdtDatetime:
			settingP->shouldShowCalendar = ! settingP->shouldShowCalendar;
			FrmUpdateForm(frmPhoneUI, FORM_UPDATE_ALL);
			isEventHandled = true;
			break;
		case ptriCategory:
			CtlHitControl(globeP->ptriCategoryP);
			isEventHandled = true;
			break;
		case gdtIcons:
			if
			(
				!globeP->isInPhoneView
				||
				settingP->shouldShowTopIcons
			)
			{
				objectId = globeP->beginProgramInPage + globeP->currentRow * 4 + globeP->currentCol;
				launchProgram(&globeP->programListP[objectId], objectId, settingP);
				isEventHandled = true;
			}
			break;
	}

	return isEventHandled;
}

//˵¼
static Boolean menuItemEventHandler(UInt16 *formUpdateCodeP, UInt16 menuItemId, SettingType *settingP)
{
	FormType				*frmAboutP;
	Boolean					isEventHandled		= false;

	switch (menuItemId)
	{
		case menuitemPhoneON:
			PhnLibSetModulePower(globeP->hsPhoneLibRef, true);
			break;
		case menuitemPhoneOFF:
			PhnLibSetModulePower(globeP->hsPhoneLibRef, false);
			break;
		case menuitemDropNetwork:
			HsNetworkDropConnection();
			break;
		case menuitemCategory:
			programManagement(globeP, MANAGEMENT_CATEGORY, settingP);

			//ˢ·б
			LstSetListChoices(globeP->lstCategoryP, NULL, 0);
			freeGlobeCategoryList();
			buildCategoryList();
			LstSetListChoices(globeP->lstCategoryP, globeP->categoryListPP, globeP->categoryCount);
			LstSetSelection(globeP->lstCategoryP, 0);
			LstSetHeight(globeP->lstCategoryP, globeP->categoryCount);
			LstSetPosition(globeP->lstCategoryP, 1, 157 - 11 * globeP->categoryCount);
			CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
			settingP->lastCategory = ALL_CATEGORIES;

			if (! globeP->isInPhoneView)
			{
				viewSwitch(VIEW_MODE_PROGRAM, settingP);
			}
			(*formUpdateCodeP) = FORM_UPDATE_ALL;
			isEventHandled = true;
			break;
		case menuitemCategoryAndShortcut:
			programManagement(globeP, MANAGEMENT_CATEGORY_AND_SHORT_CUT, settingP);
			if (globeP->isInPhoneView)
			{
				viewSwitch(VIEW_MODE_PHONE, settingP);
			}
			else
			{
				viewSwitch(VIEW_MODE_PROGRAM, settingP);
			}
			(*formUpdateCodeP) = FORM_UPDATE_ALL;
			isEventHandled = true;
			break;
		case menuitemLaunchDefaultLauncher:
			LaunchWithCommand('appl', sysFileCLauncher, sysAppLaunchCmdNormalLaunch, NULL);
			isEventHandled = true;
			break;
		case menuitemSetting:
			showSettingUI(settingP, globeP);

			(*formUpdateCodeP) = FORM_UPDATE_REFRESH;
			isEventHandled = true;
			break;
		case menuitemPreference:
			LaunchWithCommand('appl', PREF_CREATOR, sysAppLaunchCmdNormalLaunch, NULL);
			isEventHandled = true;
			break;
		case menuitemRefreshProgramList:
			buildProgramList(BUILD_LIST_MODE_REFRESH, true);

			//ˢ½
			if (globeP->isInPhoneView)
			{
				viewSwitch(VIEW_MODE_PHONE, settingP);
			}
			else
			{
				LstSetSelection(globeP->lstCategoryP, 0);
				CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
				settingP->lastCategory = ALL_CATEGORIES;
				viewSwitch(VIEW_MODE_PROGRAM, settingP);
			}
			break;
		case menuitemRebuildAll:
			if (FrmCustomAlert(alertConfirmation, STRING_REBUILD_LIST_CONFIRM, "", "") == 0)
			{
				buildProgramList(BUILD_LIST_MODE_INIT, true);

				LstSetListChoices(globeP->lstCategoryP, NULL, 0);
				freeGlobeCategoryList();
				buildCategoryList();
				LstSetListChoices(globeP->lstCategoryP, globeP->categoryListPP, globeP->categoryCount);
				LstSetSelection(globeP->lstCategoryP, 0);
				LstSetHeight(globeP->lstCategoryP, globeP->categoryCount);
				LstSetPosition(globeP->lstCategoryP, 1, 157 - 11 * globeP->categoryCount);
				CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
				settingP->lastCategory = ALL_CATEGORIES;

				//ˢ½
				if (globeP->isInPhoneView)
				{
					viewSwitch(VIEW_MODE_PHONE, settingP);
				}
				else
				{
					LstSetSelection(globeP->lstCategoryP, 0);
					CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
					viewSwitch(VIEW_MODE_PROGRAM, settingP);
				}
			}
			isEventHandled = true;
			break;
		case menuitemResetSystem:
			if (FrmCustomAlert(alertConfirmation, STRING_CONFIRM_RESET, "", "") == 0)
			{
				SysReset();
			}
			break;
		case menuitemAbout:
			frmAboutP = FrmInitForm(frmAbout);
			FrmDoDialog(frmAboutP);
			FrmDeleteForm(frmAboutP);
			(*formUpdateCodeP) = FORM_UPDATE_ALL;
			isEventHandled = true;
			break;
	}

	return isEventHandled;
}

//绰绰¼
static Boolean frmPhoneUICustomEventHandler(EventType *eventP, SettingType *settingP)
{
	Boolean			isCustomEventHandled	= false;

	switch (eventP->eType)
	{
		case PHONE_EVENT_NETWORK_SEARCHING:
		case PHONE_EVENT_POWER_ON:
			globeP->isPhoneOn = true;
			StrCopy(globeP->operatorNameP, STRING_NETWORK_SEARCHING);
			drawPhoneUI(FORM_UPDATE_STATUS_BAR_ONLY, settingP);
			isCustomEventHandled = true;
			break;
		case PHONE_EVENT_POWER_OFF:
			globeP->isPhoneOn = false;
			StrCopy(globeP->operatorNameP, STRING_PHONE_OFF);
			drawPhoneUI(FORM_UPDATE_STATUS_BAR_ONLY, settingP);
			isCustomEventHandled = true;
			break;
//		case PHONE_EVENT_DATA_CONNECTED:
//			globeP->isDataConnected = true;
//			break;
//		case PHONE_EVENT_DATA_DISCONNECTED:
//			globeP->isDataConnected = false;
//			break;
		case PHONE_EVENT_HOME:
			getOperatorName(settingP);
			drawPhoneUI(FORM_UPDATE_STATUS_BAR_ONLY, settingP);
			isCustomEventHandled = true;
			break;
		case PHONE_EVENT_OTHER:
			isCustomEventHandled = true;
			break;
		case NOTIFY_EVENT_DB:
			isCustomEventHandled = true;
			break;
		case NOTIFY_EVENT_CARD_MOUNT:
			globeP->isVolumeMounted = true;
			if (settingP->shouldAutoUpdateProgramList)
			{
				buildProgramList(BUILD_LIST_MODE_MOUNT, true);
			}
			//ˢ½
			if (globeP->isInPhoneView)
			{
				viewSwitch(VIEW_MODE_PHONE, settingP);
			}
			else
			{
				LstSetSelection(globeP->lstCategoryP, 0);
				CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
				settingP->lastCategory = ALL_CATEGORIES;
				viewSwitch(VIEW_MODE_PROGRAM, settingP);
			}
			isCustomEventHandled = true;
			break;
		case NOTIFY_EVENT_CARD_UNMOUNT:
			globeP->isVolumeMounted = false;			
		 	if (settingP->shouldAutoUpdateProgramList)
			{
				buildProgramList(BUILD_LIST_MODE_UNMOUNT, true);
			}
			//ˢ½
			if (globeP->isInPhoneView)
			{
				viewSwitch(VIEW_MODE_PHONE, settingP);
			}
			else
			{
				LstSetSelection(globeP->lstCategoryP, 0);
				CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
				settingP->lastCategory = ALL_CATEGORIES;
				viewSwitch(VIEW_MODE_PROGRAM, settingP);
			}
			isCustomEventHandled = true;
			break;
		case NOTIFY_EVENT_WAKE_UP:
			drawPhoneUI(FORM_UPDATE_CONTENT, settingP);
			break;
	}

	if (isCustomEventHandled)
	{
		FrmUpdateForm(frmPhoneUI, FORM_UPDATE_STATUS_BAR_ONLY);
	}

	return isCustomEventHandled;
}

//绰¼
static Boolean frmPhoneUIAppEventHandler(EventType *eventP, SettingType *settingP)
{
	UInt8			row, column;
	Int32			i;
	UInt16			selectedProgram;
	Boolean			isEventHandled		= false;
	UInt16			formUpdateCode		= FORM_UPDATE_NO_UPDATE;

	switch (eventP->eType)
	{
		case frmLoadEvent:
			if (eventP->data.frmLoad.formID == frmPhoneUI)
			{
				//봰
				globeP->frmPhoneUIP = FrmInitForm(frmPhoneUI);
				globeP->lstCategoryP = (ListType *)GET_OBJECT_POINTER(globeP->frmPhoneUIP, lstCategory);
				globeP->ptriCategoryP = (ControlType *)GET_OBJECT_POINTER(globeP->frmPhoneUIP, ptriCategory);

				isEventHandled = true;
			}
			break;
		case frmOpenEvent:
			if (eventP->data.frmOpen.formID == frmPhoneUI)
			{
				//ȡͼ
				if (settingP->shouldShowBackgd)
				{
					globeP->bmpBackgdP = getBackgdBitmap(settingP);
				}

				//Gadget
//				PmSysGadgetStatusGadgetTypeSet(globeP->pmGadgetLibRef, (void *)globeP->frmPhoneUIP, gdtBlueTooth, pmSysGadgetStatusGadgetBt);
//				PmSysGadgetStatusGadgetTypeSet(globeP->pmGadgetLibRef, (void *)globeP->frmPhoneUIP, gdtSignal, pmSysGadgetStatusGadgetSignal);
//				PmSysGadgetStatusGadgetTypeSet(globeP->pmGadgetLibRef, (void *)globeP->frmPhoneUIP, gdtBattery, pmSysGadgetStatusGadgetBattery);

				//ȡʶ
				getOperatorName(settingP);

				//
				FrmSetActiveForm(globeP->frmPhoneUIP);

				//
				FrmDrawForm(globeP->frmPhoneUIP);

				//ûͼģʽ
				WinSetDrawMode(winOverlay);

				//ʼ
				globeP->focusedObjectId = gdtIcons;
				globeP->isInPhoneView = true;
				getProgramListInCategory(ALL_CATEGORIES);
				globeP->beginProgramInPage = 0;
				globeP->endProgramInPage = 3;
				globeP->currentRow = 0;
				globeP->currentCol = 0;
				buildCategoryList();
				LstSetListChoices(globeP->lstCategoryP, globeP->categoryListPP, globeP->categoryCount);
				/*if (settingP->lastCategory == ALL_CATEGORIES)	//ȫ
				{
					LstSetSelection(globeP->lstCategoryP, 0);
					CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
				}
				else if (settingP->lastCategory == CARD_CATEGORY)	//濨
				{
					if (isCardInserted())
					{
						LstSetSelection(globeP->lstCategoryP, globeP->categoryCount - 1);
						CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, globeP->categoryCount - 1));
					}
					else
					{
						LstSetSelection(globeP->lstCategoryP, 0);
						CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
					}
				}
				else	//блȡ
				{
					LstSetSelection(globeP->lstCategoryP, settingP->lastCategory + 1);
					CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, settingP->lastCategory + 1));
				}*/
				column = 0;
				for (row = 0; row < globeP->categoryCount; row ++)
				{
					if (settingP->lastCategory == globeP->categoryListMappingP[row])	//ҵ
					{
						column = 1;
						LstSetSelection(globeP->lstCategoryP, row);
						CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, row));
					}
				}
				if (column == 0)	//ûƥ
				{
					settingP->lastCategory = ALL_CATEGORIES;
					LstSetSelection(globeP->lstCategoryP, 0);
					CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
				}

				LstSetHeight(globeP->lstCategoryP, globeP->categoryCount);
				LstSetPosition(globeP->lstCategoryP, 1, 157 - 11 * globeP->categoryCount);

				//ȡб
				getPluginList(&globeP->pluginListP, &globeP->pluginListCount, true);

				//ڻлͼ
				WinSetDrawWindow(globeP->offscreenWindowH);

				//Ʊ
				drawPhoneUIBackground(settingP);

				//
				drawPhoneUIContent(settingP);

				//
				if
				(
					!globeP->isInPhoneView
					||
					settingP->shouldShowTopIcons
				)
					drawPhoneUILaunchBar(settingP);

				//״̬
				drawPhoneUIStatusBar(settingP);

				//Ļ
				WinSetDrawWindow((WinHandle)globeP->frmPhoneUIP);
				WinCopyRectangle(globeP->offscreenWindowH, (WinHandle)globeP->frmPhoneUIP, &globeP->backgdRectangle, 0, 0, winPaint);

				WinSetBackColorRGB(&settingP->formFillColor, NULL);

				globeP->isFormLoaded = true;
				isEventHandled = true;
			}
			break;
		case frmUpdateEvent:
			if (eventP->data.frmUpdate.formID == frmPhoneUI)
			{
				drawPhoneUI(eventP->data.frmUpdate.updateCode, settingP);

				isEventHandled = true;
			}
			break;
		case frmObjectFocusTakeEvent:
			if (eventP->data.frmObjectFocusTake.formID == frmPhoneUI)
			{
				HsNavRemoveFocusRing(globeP->frmPhoneUIP);
				switch (eventP->data.frmObjectFocusTake.objectID)
				{
					case gdtIcons:
						break;
					default:
						HsNavDrawFocusRing
						(
							globeP->frmPhoneUIP,
							globeP->focusedObjectId,
							hsNavFocusRingNoExtraInfo,
							getFocusedObjectBounds(eventP->data.frmObjectFocusTake.objectID),
							hsNavFocusRingStyleSquare,
							true
						);
						break;
				}
				isEventHandled = true;
			}
			break;
		case menuOpenEvent:
			showHideMenuitem();
			globeP->isKeyDownDectected = false;
			break;
		case menuEvent:
			isEventHandled = menuItemEventHandler(&formUpdateCode, eventP->data.menu.itemID, settingP);
			/*if (formUpdateCode != FORM_UPDATE_NO_UPDATE)
			{
				HsNavObjectTakeFocus(globeP->frmPhoneUIP, gdtIcons);
			}*/
			HsNavObjectTakeFocus(globeP->frmPhoneUIP, globeP->focusedObjectId);
			break;
//		case frmGadgetEnterEvent:
//			if (eventP->data.gadgetEnter.gadgetID == gdtBattery)
//			{
//				globeP->isGadgetActived = true;
//			}
//			break;
		case winEnterEvent:
			if (eventP->data.winEnter.enterWindow == (WinHandle)globeP->frmPhoneUIP /* || globeP->isGadgetActived */)
			{
				formUpdateCode = FORM_UPDATE_ALL;
				globeP->isMenuActived = false;
			}
//			globeP->isGadgetActived = false;
			break;
		case popSelectEvent:
			if (eventP->data.popSelect.selection != noListSelection)
			{
				if (settingP->shouldSkipEmptyCategory && globeP->isInSearchMode == false && globeP->isPopSelectEventGeneratedByKey)
				{
					globeP->isPopSelectEventGeneratedByKey = false;
					i = eventP->data.popSelect.selection;

					do
					{
						LstSetSelection(globeP->lstCategoryP, i);
						eventP->data.popSelect.selection = i;
						settingP->lastCategory = getCategoryId(i, globeP->categoryCount);
						getProgramListInCategory(settingP->lastCategory);

						i ++;
						if (i >= globeP->categoryCount)
						{
							if (settingP->shouldSkipAllCategory)
							{
								i = 1;
							}
							else
							{
								i = 0;
							}
						}
					}while (globeP->programListCount == 0);

					CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, LstGetSelection(globeP->lstCategoryP)));
				}
				else
				{
					CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, eventP->data.popSelect.selection));
					settingP->lastCategory = getCategoryId(eventP->data.popSelect.selection, globeP->categoryCount);

					if (globeP->isInSearchMode)
					{
						if (globeP->searchKeywordNum != 0)
							searchProgramInCategory(globeP->searchKeyword, settingP->lastCategory);
					}
					else
					{
						getProgramListInCategory(settingP->lastCategory);
					}
				}
				globeP->currentRow = 0;
				globeP->currentCol = 0;
				globeP->beginProgramInPage = 0;
				globeP->endProgramInPage = 11;
				if (globeP->programListCount == 0)
				{
					globeP->endProgramInPage = 0;
				}
				else if (globeP->endProgramInPage > globeP->programListCount - 1)
				{
					globeP->endProgramInPage = globeP->programListCount - 1;
				}
				formUpdateCode = FORM_UPDATE_ALL;

				if (globeP->programListCount != 0)
				{
					globeP->focusedObjectId = gdtIcons;
					HsNavObjectTakeFocus(globeP->frmPhoneUIP, gdtIcons);
				}
			}
			break;
		case fldEnterEvent:
			break;
		case fldChangedEvent:
			break;
		case penDownEvent:
			globeP->isPenDownDectected = true;
			if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtCarrier)))
			{
				globeP->penDownObjectId = gdtCarrier;
				isEventHandled = true;
			}
			else if (globeP->isInPhoneView && RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtDatetime)))
			{
				globeP->penDownObjectId = gdtDatetime;
				isEventHandled = true;
			}
			else if
			(
				(
					!globeP->isInPhoneView
					||
					settingP->shouldShowTopIcons
				)
				&&
				isPenDownOnIcons(eventP->screenX, eventP->screenY, &selectedProgram, &row, &column))	//ƶ
			{
				globeP->currentRow = row;
				globeP->currentCol = column;
				globeP->focusedObjectId = gdtIcons;
				globeP->penDownObjectId = gdtIcons;
				formUpdateCode = FORM_UPDATE_CONTENT;
				isEventHandled = true;
			}
			break;
		case penUpEvent:
			if (globeP->isPenDownDectected)
			{
				globeP->isPenDownDectected = false;

				if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtCarrier)) &&
					globeP->penDownObjectId == gdtCarrier)		//˵
				{
					isEventHandled = controlSelectEventHandler(gdtCarrier, settingP);
				}
				else if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtDatetime)) && globeP->isInPhoneView &&
						globeP->penDownObjectId == gdtDatetime)	//
				{
					if (settingP->shouldShowDateTime)
					{
						isEventHandled = controlSelectEventHandler(gdtDatetime, settingP);
					}
				}
				else if (isPenDownOnIcons(eventP->screenX, eventP->screenY, &selectedProgram, &row, &column) &&
						globeP->penDownObjectId == gdtIcons)	//ѡ˳
				{
					if (row == globeP->currentRow && column == globeP->currentCol)
					{
						launchProgram(&globeP->programListP[selectedProgram], selectedProgram, settingP);
					}
				}
				else if (globeP->focusedObjectId == gdtIcons)		//ƶл
				{
					if (eventP->screenX >= 77 && eventP->screenX <= 82)
					{
						if (eventP->screenY >= 146 && eventP->screenY <= 152 && (! globeP->isInPhoneView))	//
						{
							if (globeP->programListCount > 0)
							{
								if (globeP->currentRow > 0)	//Уһ
								{
									globeP->currentRow --;
									formUpdateCode = FORM_UPDATE_LAUNCH_BAR_ONLY;
								}
								else if (globeP->beginProgramInPage > 0)	//Уδͷбһ
								{
									globeP->beginProgramInPage -= 4;
									globeP->endProgramInPage = globeP->beginProgramInPage + 11;
									if (globeP->endProgramInPage > globeP->programListCount - 1)	//ûҪж
									{
										globeP->endProgramInPage = globeP->programListCount - 1;
									}
									formUpdateCode = FORM_UPDATE_LAUNCH_BAR_ONLY;
								}
								else	//УѾͷָ绰ģʽ
								{
									viewSwitch(VIEW_MODE_PHONE, settingP);
									formUpdateCode = FORM_UPDATE_ALL;
								}
							}
							else	//ݣָ绰ģʽ
							{
								viewSwitch(VIEW_MODE_PHONE, settingP);
								formUpdateCode = FORM_UPDATE_ALL;
							}
							isEventHandled = true;
						}
						else if (eventP->screenY >= 153)	//
						{
							if (globeP->isInPhoneView)	//绰棬л
							{
								viewSwitch(VIEW_MODE_PROGRAM, settingP);
								formUpdateCode = FORM_UPDATE_ALL;
								isEventHandled = true;
							}
							else if (globeP->programListCount > 0)	//
							{
								if (globeP->currentRow < (globeP->endProgramInPage - globeP->beginProgramInPage) / 4)	//ĩУһ
								{
									globeP->currentRow ++;
									formUpdateCode = FORM_UPDATE_LAUNCH_BAR_ONLY;
								}
								else if (globeP->endProgramInPage < globeP->programListCount - 1)	//ĩУδͷбһ
								{
									globeP->beginProgramInPage += 4;
									globeP->endProgramInPage += 4;
									if (globeP->endProgramInPage > globeP->programListCount - 1)
									{
										globeP->endProgramInPage = globeP->programListCount - 1;
									}
									formUpdateCode = FORM_UPDATE_LAUNCH_BAR_ONLY;
								}
								if (globeP->beginProgramInPage + 4 * globeP->currentRow + globeP->currentCol > globeP->programListCount - 1)
								{
									globeP->currentCol = (globeP->endProgramInPage - globeP->beginProgramInPage) % 4;
								}
								isEventHandled = true;
							}
						}
					}
				}

				if (! isEventHandled)
				{
					isEventHandled = shouldPenDownActivePlugin(eventP);
				}

				globeP->penDownObjectId = 0;
			}
			break;
	}

	if (formUpdateCode != FORM_UPDATE_NO_UPDATE)
	{
		FrmUpdateForm(frmPhoneUI, formUpdateCode);
	}
	return isEventHandled;
}

//´
static Boolean arrowKeyEventHandler(WChar chrKey, UInt16 *formUpdateCodeP, SettingType *settingP)
{
	switch(chrKey)
	{
		case chrUpArrow:
		if (globeP->isInPhoneView)	//绰ģʽ
		{
			if (globeP->focusedObjectId == gdtIcons && settingP->shouldShowDateTime)
			{
				globeP->focusedObjectId = gdtDatetime;
				*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
				HsNavObjectTakeFocus(globeP->frmPhoneUIP, globeP->focusedObjectId);
			}
		}
		else	//ǵ绰ģʽ
		{
			if (globeP->focusedObjectId == gdtIcons)
			{
				if (globeP->programListCount > 0)
				{
					if (globeP->currentRow > 0)	//Уһ
					{
						globeP->currentRow --;
						*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
					}
					else if (globeP->beginProgramInPage > 0)	//Уδͷбһ
					{
						globeP->beginProgramInPage -= 4;
						globeP->endProgramInPage = globeP->beginProgramInPage + 11;
						if (globeP->endProgramInPage > globeP->programListCount - 1)	//ûҪж
						{
							globeP->endProgramInPage = globeP->programListCount - 1;
						}
						*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
					}
					else	//УѾͷָ绰ģʽ
					{
						viewSwitch(VIEW_MODE_PHONE, settingP);
						*formUpdateCodeP = FORM_UPDATE_ALL;
					}
				}
				else	//ݣָ绰ģʽ
				{
					viewSwitch(VIEW_MODE_PHONE, settingP);
					*formUpdateCodeP = FORM_UPDATE_ALL;
				}
			}
			else
			{
				if (globeP->programListCount > 0)
				{
					globeP->focusedObjectId = gdtIcons;
					HsNavObjectTakeFocus(globeP->frmPhoneUIP, gdtIcons);
					*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
				}
				else	//ָ绰ģʽ
				{
					viewSwitch(VIEW_MODE_PHONE, settingP);
					*formUpdateCodeP = FORM_UPDATE_ALL;
				}
			}
		}
		return true;
			
		case chrDownArrow:
		if (globeP->isInPhoneView)	//绰
		{
			switch (globeP->focusedObjectId)
			{
				case gdtDatetime:
					globeP->focusedObjectId = gdtIcons;
					*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
					break;
				case gdtIcons:
					viewSwitch(VIEW_MODE_PROGRAM, settingP);
					*formUpdateCodeP = FORM_UPDATE_ALL;
					break;
				default:
					globeP->focusedObjectId = gdtDatetime;
					break;
			}
			HsNavObjectTakeFocus(globeP->frmPhoneUIP, globeP->focusedObjectId);
		}
		else
		{
			if (globeP->programListCount > 0 && globeP->focusedObjectId == gdtIcons)
			{
				if (! globeP->isInPhoneView)	//ǵ绰ģʽ
				{
					if (globeP->currentRow < (globeP->endProgramInPage - globeP->beginProgramInPage) / 4)	//ĩУһ
					{
						globeP->currentRow ++;
						*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
					}
					else if (globeP->endProgramInPage < globeP->programListCount - 1)	//ĩУδͷбһ
					{
						globeP->beginProgramInPage += 4;
						globeP->endProgramInPage += 4;
						if (globeP->endProgramInPage > globeP->programListCount - 1)
						{
							globeP->endProgramInPage = globeP->programListCount - 1;
						}
						*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
					}
					if (globeP->beginProgramInPage + 4 * globeP->currentRow + globeP->currentCol > globeP->programListCount - 1)
					{
						globeP->currentCol = (globeP->endProgramInPage - globeP->beginProgramInPage) % 4;
					}
				}
				else	//绰ģʽתģʽ
				{
					viewSwitch(VIEW_MODE_PROGRAM, settingP);
					*formUpdateCodeP = FORM_UPDATE_ALL;
				}
			}
		}
		return true;

		case chrLeftArrow:
		if (globeP->focusedObjectId == gdtIcons)
		{
			if
			(
				!globeP->isInPhoneView
				||
				settingP->shouldShowTopIcons
			)
			{
				if (globeP->programListCount > 0)
				{
					if (globeP->currentCol > 0)	//
					{
						globeP->currentCol --;
						*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
					}
					else if (globeP->isInPhoneView) //ﲻжshouldShowtopIcons̶ʾ8
					{
						if (globeP->endProgramInPage < globeP->programListCount - 1)
						{
							globeP->currentCol = 3;
							if (globeP->beginProgramInPage == 0)
							{
								globeP->beginProgramInPage = 4;
								globeP->endProgramInPage = 8;
							}
							else
							{
								globeP->beginProgramInPage = 0;
								globeP->endProgramInPage = 4;
							}
							if (globeP->endProgramInPage > globeP->programListCount - 1)
							{
								globeP->endProgramInPage = globeP->programListCount - 1;
							}
							*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
						}
					}
					else	//ףҲڵ绰ģʽ
					{
						if (globeP->currentRow != 0)	//УҪƶб
						{
							globeP->currentCol = 3;
							globeP->currentRow --;
						}
						else if (globeP->beginProgramInPage != 0) //УҪƶб
						{
							globeP->beginProgramInPage -= 4;
							globeP->endProgramInPage = globeP->beginProgramInPage + 11;
							if (globeP->endProgramInPage > globeP->programListCount - 1)	//ûҪж
							{
								globeP->endProgramInPage = globeP->programListCount - 1;
							}
							globeP->currentCol = 3;
						}
						*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
					}
				}
				return true;
			}
		}
		return false;
		
		case chrRightArrow:
		if (globeP->focusedObjectId == gdtIcons)
		{
			if
			(
				!globeP->isInPhoneView
				||
				settingP->shouldShowTopIcons
			)
			{
				if (globeP->programListCount > 0)
				{
					if (globeP->beginProgramInPage + 4 * globeP->currentRow + globeP->currentCol < globeP->programListCount - 1)	//
					{
						if (globeP->currentCol < 3)	//δĩ1λ
						{
							globeP->currentCol ++;
							*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
						}
						else if (globeP->isInPhoneView) //ﲻжshouldShowtopIcons̶ʾ8
						{
							if (globeP->endProgramInPage < globeP->programListCount - 1)
							{
								globeP->currentCol = 0;
								if (globeP->beginProgramInPage == 0)
								{
									globeP->beginProgramInPage = 4;
									globeP->endProgramInPage = 8;
								}
								else
								{
									globeP->beginProgramInPage = 0;
									globeP->endProgramInPage = 4;
								}
								if (globeP->endProgramInPage > globeP->programListCount - 1)
								{
									globeP->endProgramInPage = globeP->programListCount - 1;
								}
								*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
							}
						}
						else	//ĩҲڵ绰ģʽ
						{
							if (globeP->beginProgramInPage + 4 * globeP->currentRow + globeP->currentCol < globeP->endProgramInPage)	//Ҫƶб
							{
								//һ
								globeP->currentCol = 0;
								globeP->currentRow ++;
								*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
							}
							else if (globeP->endProgramInPage < globeP->programListCount - 1)	//ƶб
							{
								globeP->currentCol = 0;
								globeP->beginProgramInPage += 4;
								globeP->endProgramInPage += 4;
								if (globeP->endProgramInPage > globeP->programListCount - 1)
								{
									globeP->endProgramInPage = globeP->programListCount - 1;
								}
								*formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
							}
						}
					}
				}
				return true;
			}
		}
		return false;
		
		default:
		return false;	
	}
}

//¼
static Boolean frmPhoneUIKeyEventHandler(EventType *eventP, SettingType *settingP)
{
	UInt16			selectedProgram;
	Boolean			isShortCutExist;
	Boolean			isKeyHandled		= false;

	if (! globeP->isMenuActived)
	{
		WChar			chr;
		UInt16			formUpdateCode	= FORM_UPDATE_NO_UPDATE;

		if (eventP->eType == keyDownEvent)
		{
			chr = keyTransfer(eventP->data.keyDown.chr, eventP->data.keyDown.keyCode, eventP->data.keyDown.modifiers);
			globeP->isKeyDownDectected = true;

			if (! (eventP->data.keyDown.modifiers & autoRepeatKeyMask))
			{
				if (globeP->isInPhoneView || (! settingP->shouldActivePluginInPhoneViewOnly))
				{
					isKeyHandled = shouldKeyActivePlugin(chr);
				}

				if (! isKeyHandled)
				{
					switch (chr)
					{
						case keyHard1:		//ż
							if (! globeP->isInPhoneView)	//棬л绰
							{
								viewSwitch(VIEW_MODE_PHONE, settingP);
								formUpdateCode = FORM_UPDATE_ALL;
								isKeyHandled = true;
							}
							break;
						case keyHard4:		//680
							if ((settingP->deviceModel >= TREO_680) && settingP->useAsDefaultLauncher &&
								(! (eventP->data.keyDown.modifiers & autoRepeatKeyMask)))
							{
								if (globeP->isInPhoneView)	//绰棬л
								{
									viewSwitch(VIEW_MODE_PROGRAM, settingP);
									formUpdateCode = FORM_UPDATE_ALL;
								}
								else if (CtlEnabled(globeP->ptriCategoryP))	//棬л
								{
									eventP->data.popSelect.priorSelection = LstGetSelection(globeP->lstCategoryP);
									if (eventP->data.popSelect.priorSelection != noListSelection && eventP->data.popSelect.priorSelection < LstGetNumberOfItems(globeP->lstCategoryP) - 1)
									{
										eventP->data.popSelect.selection = eventP->data.popSelect.priorSelection + 1;
									}
									else
									{
										if (settingP->shouldSkipAllCategory)
										{
											eventP->data.popSelect.selection = 1;
										}
										else
										{
											eventP->data.popSelect.selection = 0;
										}
									}
									LstSetSelection(globeP->lstCategoryP, eventP->data.popSelect.selection);
									eventP->eType = popSelectEvent;
									eventP->data.popSelect.controlID = ptriCategory;
									eventP->data.popSelect.controlP = globeP->ptriCategoryP;
									eventP->data.popSelect.listID = lstCategory;
									eventP->data.popSelect.listP = globeP->lstCategoryP;

									EvtAddEventToQueue(eventP);

									globeP->isPopSelectEventGeneratedByKey = true;
								}
								isKeyHandled = true;
							}
							break;
						case keyLaunch:		//
							if (settingP->useAsDefaultLauncher)
							{
								if (globeP->isInPhoneView)	//绰棬л
								{
									viewSwitch(VIEW_MODE_PROGRAM, settingP);
									formUpdateCode = FORM_UPDATE_ALL;
								}
								else if (CtlEnabled(globeP->ptriCategoryP))	//棬л
								{
									eventP->data.popSelect.priorSelection = LstGetSelection(globeP->lstCategoryP);
									if (eventP->data.popSelect.priorSelection != noListSelection && eventP->data.popSelect.priorSelection < LstGetNumberOfItems(globeP->lstCategoryP) - 1)
									{
										eventP->data.popSelect.selection = eventP->data.popSelect.priorSelection + 1;
									}
									else
									{
										if (settingP->shouldSkipAllCategory)
										{
											eventP->data.popSelect.selection = 1;
										}
										else
										{
											eventP->data.popSelect.selection = 0;
										}
									}
									LstSetSelection(globeP->lstCategoryP, eventP->data.popSelect.selection);
									eventP->eType = popSelectEvent;
									eventP->data.popSelect.controlID = ptriCategory;
									eventP->data.popSelect.controlP = globeP->ptriCategoryP;
									eventP->data.popSelect.listID = lstCategory;
									eventP->data.popSelect.listP = globeP->lstCategoryP;

									EvtAddEventToQueue(eventP);

									globeP->isPopSelectEventGeneratedByKey = true;
								}
								isKeyHandled = true;
							}
							break;
						case hsKeySymbol:
							if (! globeP->isInPhoneView)
							{
								CtlHitControl(globeP->ptriCategoryP);
								isKeyHandled = true;
							}
							break;
						case chrUpArrow:		//
						case vchrPageUp:
						case vchrRockerUp:
							isKeyHandled = arrowKeyEventHandler(chrUpArrow, &formUpdateCode, settingP);
							break;
						case chrDownArrow:		//
						case vchrPageDown:
						case vchrRockerDown:
							isKeyHandled = arrowKeyEventHandler(chrDownArrow, &formUpdateCode, settingP);
							break;
						case chrLeftArrow:		//
						case vchrRockerLeft:
							isKeyHandled = arrowKeyEventHandler(chrLeftArrow, &formUpdateCode, settingP);
							break;
						case chrRightArrow:		//
						case vchrRockerRight:
							isKeyHandled = arrowKeyEventHandler(chrRightArrow, &formUpdateCode, settingP);
							break;
						case vchrRockerCenter:
							if ((settingP->deviceModel == TREO_600 || settingP->deviceModel == NOT_A_TREO) || (! (eventP->data.keyDown.modifiers & willSendUpKeyMask)))
							{
								isKeyHandled = controlSelectEventHandler(globeP->focusedObjectId, settingP);
							}
							break;
						case vchrMenu:	//ʾһЩ˵Ŀ
							if (eventP->data.keyDown.chr == vchrMenu && eventP->data.keyDown.keyCode == 0 && eventP->data.keyDown.modifiers == 8)
							{
								if (MenuGetActiveMenu() != NULL)
								{
									showHideMenuitem();
								}
							}
							break;
						case keyBackspace:
							if (globeP->searchKeywordNum > 0)
							{
								globeP->searchKeywordNum--;
								globeP->searchKeyword[globeP->searchKeywordNum] = '\0';

								if (globeP->searchKeywordNum == 0)
								{
									if (globeP->isSearchBeginAtPhoneView)
									{
										viewSwitch(VIEW_MODE_PHONE, settingP);
									}
									else
									{
										viewSwitch(VIEW_MODE_PROGRAM, settingP);
									}
									formUpdateCode = FORM_UPDATE_ALL;
								}
								else
								{
									if (settingP->shouldSearchInGlobe)
									{
										searchProgramInCategory(globeP->searchKeyword, ALL_CATEGORIES);
									}
									else
									{
										searchProgramInCategory(globeP->searchKeyword, settingP->lastCategory);
									}

									globeP->currentRow = 0;
									globeP->currentCol = 0;
									globeP->beginProgramInPage = 0;
									globeP->endProgramInPage = 11;

									if (globeP->programListCount == 0)
									{
										globeP->endProgramInPage = 0;
									}
									else if (globeP->endProgramInPage > globeP->programListCount - 1)
									{
										globeP->endProgramInPage = globeP->programListCount - 1;
									}

									formUpdateCode = FORM_UPDATE_ALL;							
								}
							}
							break;
						default:
							if ((chr >= keyZero && chr <= keyNine) ||
								(eventP->data.keyDown.chr == eventP->data.keyDown.keyCode && eventP->data.keyDown.keyCode >= keyA && eventP->data.keyDown.keyCode <= keyZ) ||
								chr == keySpace)
							{
								/*{
									EventType ev;

									MemSet(&ev, sizeof(EventType), 0);
									ev.eType = keyUpEvent;
									ev.data.keyUp.chr = eventP->data.keyDown.chr;
									ev.data.keyUp.keyCode = eventP->data.keyDown.keyCode;
									ev.data.keyUp.modifiers = eventP->data.keyDown.modifiers;

									EvtAddEventToQueue(&ev);
								}*/
								isKeyHandled = true;
							}
							break;
					}
				}
			}	// not repeat key
			else
			{
				if (chr >= keyA && chr <= keyZ)
				{
					globeP->holdedKey = chr;
					drawKeyHoldTip(settingP);
				}
				else
				{
					switch(chr)
					{
						case chrUpArrow:		//
						case vchrPageUp:
						case vchrRockerUp:
							isKeyHandled = arrowKeyEventHandler(chrUpArrow, &formUpdateCode, settingP);
							break;
						case chrDownArrow:		//
						case vchrPageDown:
						case vchrRockerDown:
							isKeyHandled = arrowKeyEventHandler(chrDownArrow, &formUpdateCode, settingP);
							break;
						case chrLeftArrow:		//
						case vchrRockerLeft:
							isKeyHandled = arrowKeyEventHandler(chrLeftArrow, &formUpdateCode, settingP);
							break;
						case chrRightArrow:		//
						case vchrRockerRight:
							isKeyHandled = arrowKeyEventHandler(chrRightArrow, &formUpdateCode, settingP);
							break;
						default:
							break;
					}
				}
			}
		}
		else if (eventP->eType == keyUpEvent && globeP->isKeyDownDectected)
		{
			chr = keyTransfer(eventP->data.keyUp.chr, eventP->data.keyUp.keyCode, eventP->data.keyUp.modifiers);
			globeP->isKeyDownDectected = false;

			if (globeP->holdedKey != chrNull)	//ͷţ
			{
				if (globeP->holdedKey == chr && (chr >= keyA && chr <= keyZ))
				{
					isShortCutExist = false;
					if (! globeP->isInPhoneView)
					{
						getProgramListInCategory(ALL_CATEGORIES);	//ݼȫַΧ
					}
					for (selectedProgram = 0; selectedProgram < globeP->programListCount; selectedProgram ++)
					{
						if (eventP->data.keyHold.chr == globeP->programListP[selectedProgram].shortcut)
						{
							isShortCutExist = true;
							launchProgram(&globeP->programListP[selectedProgram], selectedProgram, settingP);
							break;
						}
					}
					if (! isShortCutExist)
					{
						FrmCustomAlert(alertInformation, STRING_SHORTCUT_NOT_DEFINED, "", "");
						if (! globeP->isInPhoneView)
						{
							getProgramListInCategory(settingP->lastCategory);	//ݼʧܣָǰб
						}
					}
				}
				globeP->holdedKey = chrNull;
			}
			/*else if (eventP->data.keyDown.keyCode == keyHard4 &&
					(settingP->deviceModel >= TREO_680 && settingP->useAsDefaultLauncher &&
					(! (eventP->data.keyDown.modifiers & autoRepeatKeyMask))))		//680
			{
				if (globeP->isInPhoneView)	//绰棬л
				{
					viewSwitch(VIEW_MODE_PROGRAM, settingP);
					formUpdateCode = FORM_UPDATE_ALL;
				}
				else if (CtlEnabled(globeP->ptriCategoryP))	//棬л
				{
					eventP->data.popSelect.priorSelection = LstGetSelection(globeP->lstCategoryP);
					if (eventP->data.popSelect.priorSelection != noListSelection && eventP->data.popSelect.priorSelection < LstGetNumberOfItems(globeP->lstCategoryP) - 1)
					{
						eventP->data.popSelect.selection = eventP->data.popSelect.priorSelection + 1;
					}
					else
					{
						if (settingP->shouldSkipAllCategory)
						{
							eventP->data.popSelect.selection = 1;
						}
						else
						{
							eventP->data.popSelect.selection = 0;
						}
					}
					LstSetSelection(globeP->lstCategoryP, eventP->data.popSelect.selection);
					eventP->eType = popSelectEvent;
					eventP->data.popSelect.controlID = ptriCategory;
					eventP->data.popSelect.controlP = globeP->ptriCategoryP;
					eventP->data.popSelect.listID = lstCategory;
					eventP->data.popSelect.listP = globeP->lstCategoryP;

					EvtAddEventToQueue(eventP);
				}
				isKeyHandled = true;
			}*/
			else if (settingP->shouldEnableProfileManagement && chr == settingP->ProfileActiveKey)		//龰ģʽ
			{
				profileEventHandler(settingP);
			}
			else if ((! EvtKeydownIsVirtual(eventP)))	//򲦴绰
			{
				if (settingP->shouldSearchProgramWhenKeyPressed)	//
				{
					if (((eventP->data.keyUp.keyCode >= keyA && eventP->data.keyUp.keyCode <= keyZ) ||
							eventP->data.keyUp.chr == keyZero || eventP->data.keyUp.keyCode == keySpace))	//
					{
						//һΰڵ绰棬ת
						if (globeP->searchKeywordNum == 0)
						{
							if (globeP->isInPhoneView)
							{
								globeP->isSearchBeginAtPhoneView = true;
								globeP->isInPhoneView = false;
								viewSwitch(VIEW_MODE_PROGRAM, settingP);
								formUpdateCode = FORM_UPDATE_ALL;
							}
							else
							{
								globeP->isSearchBeginAtPhoneView = false;
							}
						}
						
						if (globeP->searchKeywordNum < 5)
						{

							if (eventP->data.keyUp.chr < 128)
							{
								globeP->searchKeyword[globeP->searchKeywordNum++] = (Char)eventP->data.keyUp.chr;
								globeP->searchKeyword[globeP->searchKeywordNum] = '\0';
							}
							else
							{
								if (globeP->searchKeywordNum < 4)
								{
									MemMove(&globeP->searchKeyword[globeP->searchKeywordNum], &eventP->data.keyUp.chr, 2);
									globeP->searchKeywordNum += 2;
									globeP->searchKeyword[globeP->searchKeywordNum] = '\0';
								}
							}

							if (settingP->shouldSearchInGlobe)
							{
								searchProgramInCategory(globeP->searchKeyword, ALL_CATEGORIES);
							}
							else
							{
								searchProgramInCategory(globeP->searchKeyword, settingP->lastCategory);
							}

							globeP->currentRow = 0;
							globeP->currentCol = 0;
							globeP->beginProgramInPage = 0;
							globeP->endProgramInPage = 11;

							if (globeP->programListCount == 0)
							{
								globeP->endProgramInPage = 0;
							}
							else if (globeP->endProgramInPage > globeP->programListCount - 1)
							{
								globeP->endProgramInPage = globeP->programListCount - 1;
							}

							if (globeP->programListCount == 1 && settingP->shouldLaunchTheOnlyMatchedProgram)	//Ψһĳ
							{
								launchProgram(&globeP->programListP[0], 0, settingP);
							}

							globeP->isInSearchMode = true;
							formUpdateCode = FORM_UPDATE_ALL;							
						}
					}
				}
				else	//򿪵绰
				{
					if ((eventP->data.keyUp.keyCode >= keyA && eventP->data.keyUp.keyCode <= keyZ) ||
						eventP->data.keyUp.keyCode == keySpace)
					{
						EvtEnqueueKey(eventP->data.keyUp.chr, eventP->data.keyUp.keyCode, 0x0800);
					}
					else if (eventP->data.keyUp.chr == keyZero)
					{
						EvtEnqueueKey(keyZero, 0xE020, 0x0800);
					}

					shouldKeyActivePlugin(keyHard1);
				}

				isKeyHandled = true;
			}
		}

		if (formUpdateCode != FORM_UPDATE_NO_UPDATE)
		{
			FrmUpdateForm(frmPhoneUI, formUpdateCode);
		}
	}

	return isKeyHandled;
}

//绰¼
static void frmPhoneUIMainEventHandler(SettingType *settingP)
{
	EventType		event;
	Int32			i;
	UInt16			error;
	UInt16			settingSize		= 0;
	UInt16			density			= kDensityDouble;

	globeP = (GlobeType *)MemPtrNew(sizeof(GlobeType));
	MemSet(globeP, sizeof(GlobeType), 0);
	
	globeP->isVolumeMounted = isCardInserted();

	//ZLص
	globeP->ZLFuncPtr[ZLFuncPtrReopen] = NeoZLReopen;
	globeP->ZLFuncPtr[ZLFuncPtrBeginDraw] = NeoZLBeginDraw;
	globeP->ZLFuncPtr[ZLFuncPtrEndDraw] = NeoZLEndDraw;
	globeP->ZLFuncPtr[ZLFuncPtrPrepareHandlingEvent] = NeoZLPrepareHandlingEvent;
	globeP->ZLFuncPtr[ZLFuncPtrSetHint] = NeoZLSetHint;
	globeP->ZLFuncPtr[ZLFuncPtrShowHint] = NeoZLShowHint;
	globeP->ZLFuncPtr[ZLFuncPtrHideHint] = NeoZLHideHint;
	globeP->ZLFuncPtr[ZLFuncPtrPrefsReadValue] = NeoZLPrefsReadValue;
	globeP->ZLFuncPtr[ZLFuncPtrPrefsReadVariableValue] = NeoZLPrefsReadVariableValue;
	globeP->ZLFuncPtr[ZLFuncPtrPrefsWriteValue] = NeoZLPrefsWriteValue;
	globeP->ZLFuncPtr[ZLFuncPtrPrefsWriteVariableValue] = NeoZLPrefsWriteVariableValue;
	globeP->ZLFuncPtr[ZLFuncPtrRefreshBackground] = NeoZLRefreshBackground;
	globeP->ZLFuncPtr[ZLFuncPtrDrawChars] = NeoZLDrawChars;
	globeP->ZLFuncPtr[ZLFuncPtrDrawBitmap] = NeoZLDrawBitmap;

	//ȡİ汾
	i = PrefGetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, NULL, &settingSize, true);
	if (i != PREFERENCE_VERSION)	//һΰװ°汾
	{
		if (i != noPreferenceFound)		//°汾
		{
			//ɰ汾Ĳ
			PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, i, NULL, 0, true);
			i = noPreferenceFound;
		}
		else		//һΰװ
		{
			//б
			buildProgramList(BUILD_LIST_MODE_INIT, true);
		}
	}
	settingSize = sizeof(SettingType);
	if (i == noPreferenceFound)		//Ҫó
	{
		//ʼ
		MemSet(settingP, settingSize, 0);
		isRunningOnTreo(settingP);
		getDefaultColor(settingP);

		settingP->shouldSearchProgramWhenKeyPressed = true;

		settingP->shouldShowDateTime = true;
		settingP->shouldAddMask = true;

		settingP->maskPattern = grayPattern;

		settingP->shouldActivePluginInPhoneViewOnly = true;

		settingP->lastCategory = ALL_CATEGORIES;

		profileManagement(settingP, true);

		//
		PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, PREFERENCE_VERSION, settingP, settingSize, true);
	}
	//ȡ
	PrefGetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, settingP, &settingSize, true);

	//ǷдӿҪɾĳ
	deleteProgramLaunchFromCard(settingP);

	//عб
	if (i != noPreferenceFound && settingP->shouldAutoUpdateProgramList)
	{
		buildProgramList(BUILD_LIST_MODE_REFRESH, false);
	}
	
	if (! settingP->useAsDefaultLauncher)
	{
		settingP->isNotificationRegistered = false;
	}

	//ʼȫֱ
	setRectangleData();
	getSoundStatusBitmap(settingP);
	globeP->dateStringP = (Char *)MemPtrNew(64);
	globeP->timeStringP = (Char *)MemPtrNew(32);
	if (settingP->shouldAddMask)
	{
		globeP->pluginParameter.defaultDisplayEffect = DISPLAY_EFFECT_ADD_MASK;
	}
	else if (settingP->shouldAddLine)
	{
		globeP->pluginParameter.defaultDisplayEffect = DISPLAY_EFFECT_ADD_LINE;
	}
	else
	{
		globeP->pluginParameter.defaultDisplayEffect = DISPLAY_EFFECT_NONE;
	}
	globeP->pluginParameter.defaultMaskPattern = settingP->maskPattern;
	globeP->pluginParameter.textColor = settingP->textColor;
	globeP->pluginParameter.textOutlineColor = settingP->textOutlineColor;
	globeP->pluginParameter.foreColor = settingP->formFrameColor;
	globeP->pluginParameter.backgdColor = settingP->formFillColor;

	//߷ֱĻּ֧
	if (WinGetSupportedDensity(&density) != errNone)
		globeP->isHighDensitySupported = false;
	else
		globeP->isHighDensitySupported = true;

	//ýɫ
	for (i = hsNavFocusColorRing1; i <= hsNavFocusColorRing6; i ++)
	{
		HsNavSetFocusColor(i, &settingP->focusRingColor[i - hsNavFocusColorRing1], NULL);
	}

	//ͼ
	globeP->offscreenWindowH = WinCreateOffscreenWindow(160, 160, nativeFormat, &error);

	if (error == errNone)
	{
		UInt32		value	= 0;

		//Gadget
//		SysLibLoad(kPmSysGadgetLibType, kPmSysGadgetLibCreator, &globeP->pmGadgetLibRef);
//		PmSysGadgetLibOpen(globeP->pmGadgetLibRef);

		//򿪵绰⣬ע绰¼
		HsGetPhoneLibrary(&globeP->hsPhoneLibRef);
		PhnLibOpen(globeP->hsPhoneLibRef);
		if (! settingP->isPhoneLibraryRegistered)
		{
			PhnLibRegister(globeP->hsPhoneLibRef, APPLICATION_CREATOR, phnServiceAll);
			settingP->isPhoneLibraryRegistered = true;
		}

		//绰Ƿ. isPhoneOnѾʼΪ0.
		if (PhnLibModulePowered(globeP->hsPhoneLibRef) == phnPowerOn)
			globeP->isPhoneOn = true;

		//
		SysLibLoad(tonesLibType, tonesLibCreator, &globeP->tonesLibRef);
		TonesLibOpen(globeP->tonesLibRef);

		//עϢ
		if (! settingP->isNotificationRegistered)
		{
			notifyRegister(settingP);
		}

		//򿪴
		FrmGotoForm(frmPhoneUI);

		do
		{
			EvtGetEvent(&event, evtWaitForever);

			if (! frmPhoneUICustomEventHandler(&event, settingP))
			{
				if (! frmPhoneUIKeyEventHandler(&event, settingP))
				{
					if (! SysHandleEvent(&event))
					{
						if (! MenuHandleEvent(0, &event, &error))
						{
							if ((! frmPhoneUIAppEventHandler(&event, settingP)) && (! globeP->isMenuActived))
							{
								WinSetBackColorRGB(&settingP->topStatusFillColor, NULL);
								FrmDispatchEvent(&event);
								if (shouldUpdateDateTime() && globeP->isFormLoaded)
								{
									FrmUpdateForm(frmPhoneUI, FORM_UPDATE_CONTENT);
								}
							}
						}
						else
						{
							globeP->isMenuActived = true;
							globeP->isKeyDownDectected = false;
						}
					}
				}
			}
		}while (event.eType != appStopEvent);

		//ͷڴ
		freePluginList(globeP->pluginListP, globeP->pluginListCount);
		LstSetListChoices(globeP->lstCategoryP, NULL, 0);
		freeGlobeCategoryList();
		freeGlobeProgramList();
		if (globeP->bmpBackgdP)
		{
			BmpDelete((BitmapType *)globeP->bmpBackgdP);
		}

		//ر
		TonesLibClose(globeP->tonesLibRef);
		if (! settingP->useAsDefaultLauncher)
		{
			//ȡ绰¼ע
			if (settingP->isPhoneLibraryRegistered)
			{
				PhnLibRegister(globeP->hsPhoneLibRef, APPLICATION_CREATOR, 0);
				PhnLibClose(globeP->hsPhoneLibRef);
				settingP->isPhoneLibraryRegistered = false;
			}

			//ȡϢע
			notifyUnregister(settingP);
		}

		//رGadget
//		PmSysGadgetLibClose(globeP->pmGadgetLibRef);
	}

	WinDeleteWindow(globeP->offscreenWindowH, false);
	MemPtrFree(globeP->timeStringP);
	MemPtrFree(globeP->dateStringP);
	for (i = 0; i < 10; i ++)
	{
		if (globeP->bmpSoundStatusP[i] != NULL)
			MemPtrFree(globeP->bmpSoundStatusP[i]);
	}
	for (i = 0; i < 2; i ++)
	{
		if (globeP->bmpPowerStatusP[i] != NULL)
			MemPtrFree(globeP->bmpPowerStatusP[i]);
	}
	MemPtrFree(globeP);

	//
	PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, PREFERENCE_VERSION, settingP, settingSize, true);

	//رմ
	FrmCloseAllForms();
}

//绰
static void launchPhone()
{
	UInt16			settingSize	= sizeof(SettingType);
	SettingType		*settingP	= MemPtrNew(settingSize);

	if (PrefGetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, settingP, &settingSize, true) != noPreferenceFound)
	{
		PluginParameterType		*pluginParameterP	= (PluginParameterType *)MemPtrNew(sizeof(PluginParameterType));

		MemSet(pluginParameterP, sizeof(PluginParameterType), 0);
		pluginParameterP->actionType = ACTION_TYPE_KEY;
		pluginParameterP->textColor = settingP->textColor;
		pluginParameterP->textOutlineColor = settingP->textOutlineColor;
		pluginParameterP->foreColor = settingP->formFrameColor;
		pluginParameterP->backgdColor = settingP->formFillColor;

		CallWithCommand('NlPh', APPLICATION_CREATOR, LAUNCH_CODE_PLUGIN_ACTION, pluginParameterP);

		MemPtrFree(pluginParameterP);
	}

	MemPtrFree(settingP);
}

//Ϣ¼
static void notifyEventHandler(SysNotifyParamType *notifyP)
{
	EventType			customEvent;

	customEvent.eType = nilEvent;
	switch (notifyP->notifyType)
	{
		/*case sysNotifyDBCreatedEvent:
		{
			buildProgramList(BUILD_LIST_MODE_ADD_DB, notifyP->notifyDetailsP);
			customEvent.eType = NOTIFY_EVENT_DB;
			break;
		}
		case sysNotifyDBChangedEvent:
		{
			buildProgramList(BUILD_LIST_MODE_CHANGE_DB, notifyP->notifyDetailsP);
			customEvent.eType = NOTIFY_EVENT_DB;
			break;
		}
		case sysNotifyDBDeletedEvent:
		{
			buildProgramList(BUILD_LIST_MODE_DELETE_DB, notifyP->notifyDetailsP);
			customEvent.eType = NOTIFY_EVENT_DB;
			break;
		}*/
		case sysNotifyVolumeMountedEvent:
		{
			notifyP->handled |= vfsHandledUIAppSwitch;
			customEvent.eType = NOTIFY_EVENT_CARD_MOUNT;
			break;
		}
		case sysNotifyVolumeUnmountedEvent:
		{
			customEvent.eType = NOTIFY_EVENT_CARD_UNMOUNT;
			break;
		}
		case sysNotifyLateWakeupEvent:
		{
			customEvent.eType = NOTIFY_EVENT_WAKE_UP;
			break;
		}
		case sysNotifyVirtualCharHandlingEvent:
		{
			switch (((SysNotifyVirtualCharHandlingType *)notifyP->notifyDetailsP)->keyDown.keyCode)
			{
				case keyHard1:
				{
					if (((NotifyParameterType *)notifyP->userDataP)->shouldRelayKey &&
						((NotifyParameterType *)notifyP->userDataP)->isInPhone == false)
					{
						notifyP->handled = true;
						launchPhone();
					}
					break;
				}
				case keyHard4:
				{
					if ((((NotifyParameterType *)notifyP->userDataP)->deviceModel >= TREO_680) &&
						((NotifyParameterType *)notifyP->userDataP)->useAsDefaultLauncher)
					{
						notifyP->handled = true;
						LaunchWithCommand('appl', APPLICATION_CREATOR, sysAppLaunchCmdNormalLaunch, NULL);
					}
					break;
				}
			}
			break;
		}
	}

	if (customEvent.eType != nilEvent)
	{
		EvtAddUniqueEventToQueue(&customEvent, 0, false);
	}
}

#if 0
#pragma mark -
#pragma mark START UP FUNCTIONS
#pragma mark -
#endif

//豸
static Boolean isRunningOnTreo(SettingType *settingP)
{
	UInt32			companyID;
	UInt32			deviceID;

	FtrGet(sysFtrCreator, sysFtrNumOEMCompanyID, &companyID);
	FtrGet(sysFtrCreator, sysFtrNumOEMDeviceID, &deviceID);

	if ((companyID == 'Palm' || companyID == 'hspr') && deviceID == 'D053') //Treo680
	{
		settingP->deviceModel = TREO_680;
	}
	else if ((companyID == 'Palm' || companyID == 'hspr') && deviceID == 'D061') //CENTRO
	{
		settingP->deviceModel = CENTRO;
	}
	else if ((companyID == 'Palm' || companyID == 'hspr') && deviceID == 'D052') //Treo700p
	{
		settingP->deviceModel = TREO_700P;
	}
	else if ((companyID == 'Palm' || companyID == 'hspr') && deviceID == 'D060') //Treo755p
	{
		settingP->deviceModel = TREO_755P;
	}
	else if (companyID == 'hspr' && (deviceID == 'H102' || deviceID == 'H202')) //Treo650
	{
		settingP->deviceModel = TREO_650;
	}
	else if (companyID == 'hspr' && (deviceID == 'H101' || deviceID == 'H201')) //Treo600
	{
		settingP->deviceModel = TREO_600;
	}
	else
	{
		settingP->deviceModel = NOT_A_TREO;
		return false;
	}

	return true;
}

//עϢ
static void notifyUnregister(SettingType *settingP/*Boolean shouldSetVirtualCharHandlingNotify*/)
{
	UInt16					cardNo;
	LocalID					dbId;
	NotifyParameterType		*notifyParameterP;

	SysCurAppDatabase(&cardNo, &dbId);

	//SysNotifyUnregister(cardNo, dbId, sysNotifyDBCreatedEvent, sysNotifyNormalPriority);
	//SysNotifyUnregister(cardNo, dbId, sysNotifyDBChangedEvent, sysNotifyNormalPriority);
	//SysNotifyUnregister(cardNo, dbId, sysNotifyDBDeletedEvent, sysNotifyNormalPriority);
	//SysNotifyUnregister(cardNo, dbId, sysNotifyCardInsertedEvent, sysNotifyNormalPriority);
	//SysNotifyUnregister(cardNo, dbId, sysNotifyCardRemovedEvent, sysNotifyNormalPriority);
	SysNotifyUnregister(cardNo, dbId, sysNotifyVolumeMountedEvent, sysNotifyNormalPriority);
	SysNotifyUnregister(cardNo, dbId, sysNotifyVolumeUnmountedEvent, sysNotifyNormalPriority);
	SysNotifyUnregister(cardNo, dbId, sysNotifyLateWakeupEvent, sysNotifyNormalPriority);
	SysNotifyUnregister(cardNo, dbId, sysNotifyVirtualCharHandlingEvent, sysNotifyNormalPriority);

	if (FtrGet(APPLICATION_CREATOR, NOTIFY_PARAMETER_FEATURE_ID, (UInt32 *)&notifyParameterP) != errNone)
	{
		FtrPtrFree(APPLICATION_CREATOR, NOTIFY_PARAMETER_FEATURE_ID);
	}

	settingP->isNotificationRegistered = false;
}

//עϢ
static void notifyRegister(SettingType *settingP/*, Boolean shouldSetVirtualCharHandlingNotify*/)
{
	UInt16					cardNo;
	LocalID					dbId;
	PhoneSettingType		phoneSetting;
	NotifyParameterType		*notifyParameterP;
	NotifyParameterType		tmpNotifyParameter;
	UInt16					phoneSettingSize	= sizeof(PhoneSettingType);

	SysCurAppDatabase(&cardNo, &dbId);

	//SysNotifyRegister(cardNo, dbId, sysNotifyDBCreatedEvent, NULL, sysNotifyNormalPriority, NULL);
	//SysNotifyRegister(cardNo, dbId, sysNotifyDBChangedEvent, NULL, sysNotifyNormalPriority, NULL);
	//SysNotifyRegister(cardNo, dbId, sysNotifyDBDeletedEvent, NULL, sysNotifyNormalPriority, NULL);
	//SysNotifyRegister(cardNo, dbId, sysNotifyCardInsertedEvent, NULL, sysNotifyNormalPriority, NULL);
	//SysNotifyRegister(cardNo, dbId, sysNotifyCardRemovedEvent, NULL, sysNotifyNormalPriority, NULL);
	SysNotifyRegister(cardNo, dbId, sysNotifyVolumeMountedEvent, NULL, sysNotifyNormalPriority, NULL);
	SysNotifyRegister(cardNo, dbId, sysNotifyVolumeUnmountedEvent, NULL, sysNotifyNormalPriority, NULL);
	SysNotifyRegister(cardNo, dbId, sysNotifyLateWakeupEvent, NULL, sysNotifyNormalPriority, NULL);

	if (FtrGet(APPLICATION_CREATOR, NOTIFY_PARAMETER_FEATURE_ID, (UInt32 *)&notifyParameterP) != errNone)
	{
		FtrPtrNew(APPLICATION_CREATOR, NOTIFY_PARAMETER_FEATURE_ID, sizeof(NotifyParameterType), (void **)&notifyParameterP);
		FtrGet(APPLICATION_CREATOR, NOTIFY_PARAMETER_FEATURE_ID, (UInt32 *)&notifyParameterP);
	}

	if (PrefGetAppPreferences(APPLICATION_CREATOR, PHONE_PREFERENCE_ID, &phoneSetting, &phoneSettingSize, true) != noPreferenceFound)
	{
		tmpNotifyParameter.shouldRelayKey = phoneSetting.shouldRelayKey;
	}
	else
	{
		tmpNotifyParameter.shouldRelayKey = false;
	}

	tmpNotifyParameter.isInPhone = false;
	tmpNotifyParameter.deviceModel = settingP->deviceModel;
	tmpNotifyParameter.useAsDefaultLauncher = settingP->useAsDefaultLauncher;

	DmWrite(notifyParameterP, 0, &tmpNotifyParameter, sizeof(NotifyParameterType));

	SysNotifyRegister(cardNo, dbId, sysNotifyVirtualCharHandlingEvent, NULL, sysNotifyNormalPriority, notifyParameterP);

	settingP->isNotificationRegistered = true;
}

//ڵ
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	SettingType			setting;
	UInt16				i;
	PhnEventType		*phnEventP;

	switch (cmd)
	{
		case sysAppLaunchCmdNormalLaunch:
		case sysAppLaunchCmdReturnFromPanel:
			if (isRunningOnTreo(&setting))
			{
				frmPhoneUIMainEventHandler(&setting);
			}
			else
			{
				FrmCustomAlert(alertError, STRING_DEVICE_ERROR, "", "");
			}
			break;
		case sysAppLaunchCmdSystemReset:
			i = sizeof(SettingType);
			if (PrefGetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, &setting, &i, true) != noPreferenceFound)
			{
				setting.isNotificationRegistered = false;
				setting.isPhoneLibraryRegistered = false;
				PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, PREFERENCE_VERSION, &setting, sizeof(SettingType), true);
			}
			break;
		case sysAppLaunchCmdNotify:
			notifyEventHandler((SysNotifyParamType *)cmdPBP);
			break;
		case phnLibLaunchCmdEvent:
		{
			EventType				customEvent;
			NotifyParameterType		*notifyParameterP;

			phnEventP = (PhnEventType *)cmdPBP;
			MemSet(&customEvent, sizeof(EventType), 0);
			switch (phnEventP->eventType)
			{
				case phnEvtRegistration:
					switch (phnEventP->data.registration.status)
					{
						case registrationHome:
							customEvent.eType = PHONE_EVENT_HOME;
							break;
						case registrationSearch:
							customEvent.eType = PHONE_EVENT_NETWORK_SEARCHING;
							break;
					}
					break;
				case phnEvtPower:
					switch (phnEventP->data.power.state)
					{
						case phnPowerOn:
							customEvent.eType = PHONE_EVENT_POWER_ON;
							break;
						case phnPowerOff:
							customEvent.eType = PHONE_EVENT_POWER_OFF;
							break;
					}
					break;
				case phnEvtIndication:
					switch (phnEventP->data.indication.kind)
					{
						case indicationNetworkSearch:
							customEvent.eType = PHONE_EVENT_NETWORK_SEARCHING;
							break;
						case indicationStartingRadio:
							customEvent.eType = PHONE_EVENT_POWER_ON;
							break;
						case indicationPoweringOffRadio:
							customEvent.eType = PHONE_EVENT_POWER_OFF;
							break;
					}
					break;
				case phnEvtProgress:
				case phnEvtConnectInd:
				case phnEvtConnectConf:
					if (FtrGet(APPLICATION_CREATOR, NOTIFY_PARAMETER_FEATURE_ID, (UInt32 *)&notifyParameterP) == errNone)
					{
						NotifyParameterType		tmpNotifyParameter	= (*notifyParameterP);

						tmpNotifyParameter.isInPhone = true;
						DmWrite(notifyParameterP, 0, &tmpNotifyParameter, sizeof(NotifyParameterType));
					}
					break;
				case phnEvtDisconnectInd:
				case phnEvtDisconnectConf:
					i = sizeof(SettingType);
					if (PrefGetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, &setting, &i, true) != noPreferenceFound)
					{
						if (setting.shouldReturnToLauncherAfterCall)
						{
							LaunchWithCommand('appl', APPLICATION_CREATOR, sysAppLaunchCmdNormalLaunch, NULL);
						}
					}

					if (FtrGet(APPLICATION_CREATOR, NOTIFY_PARAMETER_FEATURE_ID, (UInt32 *)&notifyParameterP) == errNone)
					{
						NotifyParameterType		tmpNotifyParameter	= (*notifyParameterP);

						tmpNotifyParameter.isInPhone = false;
						DmWrite(notifyParameterP, 0, &tmpNotifyParameter, sizeof(NotifyParameterType));
					}
					break;

//				case phnEvtDataSessionStatus:
//					switch (phnEventP->data.dataSesstionStatus.sessionStatus)
//					{
//						case phnPacketDataSessionNone:
//							customEvent.eType = PHONE_EVENT_DATA_DISCONNECTED;
//							break;
//						case phnPacketDataSessionDormant:
//						case phnPacketDataSessionActive:
//							customEvent.eType = PHONE_EVENT_DATA_CONNECTED;
//							break;
//					}
//				case phnEvtDataSessionDisconnected:
//					customEvent.eType = PHONE_EVENT_DATA_DISCONNECTED;
//					break;
			}
			if (customEvent.eType != nilEvent)
			{
				//EvtAddEventToQueue(&customEvent);
				EvtAddUniqueEventToQueue(&customEvent, 0, false);
			}
			break;
		}
	}

	return errNone;
}

#if 0
#pragma mark -
#pragma mark COMMON FUNCTIONS
#pragma mark -
#endif

//λ
static void raiseProgram(ProgramListType *programP)
{
	if (! (programP->addOnInfo & PROGRAM_LIST_STATIC_MASK))	//ǹ̶
	{
		UInt16			currentProgramIndex;
		UInt16			newProgramIndex;
		UInt16			i;
		MemHandle		recordH;
		ProgramListType	*recordP;
		UInt16			lastStaticProgramIndex	= 0xFFFF;
		DmOpenRef		dbRef					= DmOpenDatabaseByTypeCreator(PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadWrite);
		UInt16			programCount			= DmNumRecordsInCategory(dbRef, dmAllCategories);

		//ȡǰбеλãԼһ̶λ
		for (i = 0; i < programCount; i ++)
		{
			recordH = DmQueryRecord(dbRef, i);
			recordP = (ProgramListType *)MemHandleLock(recordH);

			if ((recordP->addOnInfo & PROGRAM_LIST_STATIC_MASK))
			{
				lastStaticProgramIndex = i;
			}

			if (recordP->type == programP->type && recordP->creatorId == programP->creatorId)
			{
				currentProgramIndex = i;
				MemHandleUnlock(recordH);
				break;
			}

			MemHandleUnlock(recordH);
		}

		if ((lastStaticProgramIndex != 0xFFFF && currentProgramIndex - 1 > lastStaticProgramIndex) ||
			(lastStaticProgramIndex == 0xFFFF && currentProgramIndex > 0))	//Եλ
		{
			if (lastStaticProgramIndex != 0xFFFF)
			{
				newProgramIndex = currentProgramIndex - (currentProgramIndex - lastStaticProgramIndex) / 2;
			}
			else
			{
				newProgramIndex = currentProgramIndex / 2;
			}
			DmMoveRecord(dbRef, currentProgramIndex, newProgramIndex);
		}

		DmCloseDatabase(dbRef);
	}
}

//
static void launchProgram(ProgramListType *programInfoP, UInt16 selectedProgram, SettingType *settingP)
{
	//̬λ
	if (settingP->shouldAutoRaiseProgram)
		raiseProgram(programInfoP);

	if (programInfoP->VFSNameP[0] == '\0')	//ڴеĳ
	{
		LaunchWithCommand(programInfoP->type, programInfoP->creatorId, sysAppLaunchCmdNormalLaunch, NULL);
	}
	else
	{
		Err			errorCode;
		UInt16		cardNo;
		LocalID		dbId;
		UInt16		volRef 			= 0;
		UInt32		volIterator		= vfsIteratorStart;

		//ȡָ
		while (volIterator != vfsIteratorStop)
		{
			VFSVolumeEnumerate(&volRef, &volIterator);
		}

		if (volRef > 0)
		{
			StrCopy(globeP->cacheP, "/PALM/Launcher/");
			StrCat(globeP->cacheP, programInfoP->VFSNameP);
			errorCode = VFSImportDatabaseFromFile(volRef, globeP->cacheP, &cardNo, &dbId);
			if (errorCode != expErrNotEnoughPower && errorCode != vfsErrBadName)
			{
				if (errorCode != dmErrAlreadyExists)	//¼
				{
					if (DmDatabaseInfo(cardNo, dbId, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &settingP->lastProgramLaunchFromCard.type, &settingP->lastProgramLaunchFromCard.creatorId) == errNone)
					{
						PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, PREFERENCE_VERSION, settingP, sizeof(SettingType), true);
					}
				}
				SysUIAppSwitch(cardNo, dbId, sysAppLaunchCmdNormalLaunch, NULL);
			}
			else
			{
				FrmCustomAlert(alertError, STRING_LAUNCH_ERROR, "", "");
			}
		}
	}
}

//
static void paintCharWithOutline(Char chr, Coord x, Coord y, RGBColorType *outlineColorP)
{
	RGBColorType	defaultColor;

	//
	WinSetTextColorRGB(outlineColorP, &defaultColor);
	if (!globeP->isHighDensitySupported)	//߷Ļ
	{
		Coord	dX = x * 2;
		Coord	dY = y * 2;

		WinSetCoordinateSystem(kCoordinatesDouble);
		WinPaintChar(chr, dX - 1, dY - 1);
		WinPaintChar(chr, dX + 1, dY - 1);
		WinPaintChar(chr, dX - 1, dY + 1);
		WinPaintChar(chr, dX + 1, dY + 1);
		WinSetCoordinateSystem(kCoordinatesStandard);
	}
	else	//ָ֧߷ֱĻ
	{
		WinPaintChar(chr, x - 1, y - 1);
		WinPaintChar(chr, x + 1, y - 1);
		WinPaintChar(chr, x - 1, y + 1);
		WinPaintChar(chr, x + 1, y + 1);
	}
	WinSetTextColorRGB(&defaultColor, NULL);

	//ַ
	WinPaintChar(chr, x, y);
}

//ַ
static void paintCharsWithOutline(const Char *stringP, UInt16 stringLength, Coord x, Coord y, RGBColorType *outlineColorP)
{
	RGBColorType	defaultColor;

	//
	WinSetTextColorRGB(outlineColorP, &defaultColor);
	if (globeP->isHighDensitySupported)		//߷Ļ
	{
		Coord	dX = x * 2;
		Coord	dY = y * 2;

		WinSetCoordinateSystem(kCoordinatesDouble);
		WinPaintChars(stringP, stringLength, dX - 1, dY - 1);
		WinPaintChars(stringP, stringLength, dX + 1, dY - 1);
		WinPaintChars(stringP, stringLength, dX - 1, dY + 1);
		WinPaintChars(stringP, stringLength, dX + 1, dY + 1);
		WinSetCoordinateSystem(kCoordinatesStandard);
	}
	else	//ָ֧߷ֱĻ
	{
		WinPaintChars(stringP, stringLength, x - 1, y - 1);
		WinPaintChars(stringP, stringLength, x + 1, y - 1);
		WinPaintChars(stringP, stringLength, x - 1, y + 1);
		WinPaintChars(stringP, stringLength, x + 1, y + 1);
	}
	WinSetTextColorRGB(&defaultColor, NULL);

	//ַ
	WinPaintChars(stringP, stringLength, x, y);
}

//ָȵ
static void winPaintTruncChars(Char *stringP, UInt16 stringLength, Coord x, Coord y, UInt16 maxWidth, Boolean shouldDrawOutline, RGBColorType *outlineColorP)
{
	if (stringLength > 3 && stringLength < 128)
	{
		StrCopy(globeP->cacheP, stringP);
		if ((UInt8)globeP->cacheP[stringLength - 2] > 127)	//
		{
			StrCopy((globeP->cacheP + (stringLength - 2)), "...");
			stringLength ++;
		}
		else
		{
			StrCopy((globeP->cacheP + (stringLength - 1)), "...");
			stringLength += 2;
		}

		while (stringLength > 3)
		{
			if (FntCharsWidth(globeP->cacheP, stringLength) <= maxWidth)
			{
				if (shouldDrawOutline)
				{
					paintCharsWithOutline(globeP->cacheP, stringLength, x, y, outlineColorP);
				}
				else
				{
					WinPaintChars(globeP->cacheP, stringLength, x, y);
				}
				break;
			}
			else
			{
				if ((UInt8)globeP->cacheP[stringLength - 5] > 127)
				{
					StrCopy((globeP->cacheP + (stringLength - 5)), "...");
					stringLength -= 2;
				}
				else
				{
					StrCopy((globeP->cacheP + (stringLength - 4)), "...");
					stringLength --;
				}
			}
		}
	}
}

//ַתUInt32λװ
static void convertCharsToBitPackage(Char *srcStringP, UInt32 *pyTableP, UInt32 *destUInt32BitPackageP, UInt16 *UInt32BitPackageLengthP)
{
	WChar		chr;

	(*UInt32BitPackageLengthP) = 0;

	while (*srcStringP && (*UInt32BitPackageLengthP) < 5)
	{
		if (((UInt8)(*srcStringP)) > 127 && pyTableP)	//
		{
			MemMove(&chr, srcStringP, 2);
			srcStringP += 2;
			destUInt32BitPackageP[(*UInt32BitPackageLengthP)] = (pyTableP[chr - 33088] & PRONUNCIATION_COUNT);
		}
		else if (((UInt8)(*srcStringP)) >= 97 && ((UInt8)(*srcStringP)) <= 122)	//дӢ
		{
			destUInt32BitPackageP[(*UInt32BitPackageLengthP)] = (((UInt32)1) << ((*srcStringP) - 97));
			srcStringP ++;
		}
		else if (((UInt8)(*srcStringP)) >= 65 && ((UInt8)(*srcStringP)) <= 90)	//СдӢ
		{
			destUInt32BitPackageP[(*UInt32BitPackageLengthP)] = (((UInt32)1) << ((*srcStringP) - 65));
			srcStringP ++;
		}
		else	//
		{
			destUInt32BitPackageP[(*UInt32BitPackageLengthP)] = (UInt32)(*srcStringP);
			destUInt32BitPackageP[(*UInt32BitPackageLengthP)] |= 0x80000000;
			srcStringP ++;
		}
		(*UInt32BitPackageLengthP) ++;
	}
}

static Char *strPreventChr(Char *stringP, Char chr)
{
	UInt16		stringLength	= StrLen(stringP);

	for (stringLength --; stringLength > 0; stringLength --)
	{
		if (stringP[stringLength] == chr)
		{
			return stringP + stringLength;
		}
	}

	return NULL;
}

//ȫֱķб
static void freeGlobeCategoryList()
{
	if (globeP->categoryListPP)
	{
		UInt16		i;

		for (i = 0; i < globeP->categoryCount; i ++)
		{
			MemPtrFree(globeP->categoryListPP[i]);
		}
		MemPtrFree(globeP->categoryListPP);
		globeP->categoryCount = 0;
	}

	if (globeP->categoryListMappingP)
	{
		MemPtrFree(globeP->categoryListMappingP);
	}

	if (globeP->hideCategoryP)
	{
		MemPtrFree(globeP->hideCategoryP);
	}
	globeP->hideCategoryCount = 0;

	globeP->categoryListPP = NULL;
	globeP->categoryListMappingP = NULL;
	globeP->hideCategoryP = NULL;
}

//ȫֱĳб
static void freeGlobeProgramList()
{
	if (globeP->programListP != NULL)
	{
		MemPtrFree(globeP->programListP);

		globeP->programListCount = 0;
		globeP->programListP = NULL;
	}
}

//ֵת
static WChar keyTransfer(WChar chr, UInt16 keyCode, UInt16 modifiers)
{
	if (modifiers & 0x0008)
	{
		if (keyCode != 0)
		{
			return (WChar)keyCode;
		}
		else
		{
			return chr;
		}
	}
	else
	{
		return chr;
	}
}

//Զ尴
static WChar customKey()
{
	FormType	*frmP;
	EventType	event;
	WChar		key		= chrNull;

	FrmPopupForm(frmSetKeyTips);

	do
	{
		EvtGetEvent(&event, evtWaitForever);

		if (event.eType == keyUpEvent)
		{
			if (event.data.keyDown.keyCode != 0)
			{
				key = (WChar)event.data.keyDown.keyCode;
			}
			else
			{
				key = (WChar)event.data.keyDown.chr;
			}
		}
		else if (event.eType == keyDownEvent && (! (event.data.keyDown.modifiers & 0x0008)))
		{
			key = event.data.keyDown.chr;
		}

		if (key == chrNull && (event.eType != keyUpEvent && event.eType != keyDownEvent))
		{
			if (! SysHandleEvent(&event))
			{
				if (event.eType == frmLoadEvent && event.data.frmLoad.formID == frmSetKeyTips)
				{
					frmP = FrmInitForm(frmSetKeyTips);
					FrmSetActiveForm(frmP);
				}
				else if (event.eType == frmOpenEvent && event.data.frmOpen.formID == frmSetKeyTips)
				{
					FrmDrawForm(frmP);
				}
				else
				{
					FrmDispatchEvent(&event);
				}
			}
		}
	}while (key == chrNull && event.eType != appStopEvent);

	//˳
	FrmReturnToForm(0);

	return key;
}

//ɫͬȵĻɫ
static void getGrayColor(RGBColorType *grayColor, RGBColorType *originalColor)
{
	UInt32		y	= (((UInt32)originalColor->r) * 299 + ((UInt32)originalColor->g) * 587 + ((UInt32)originalColor->b) * 114) / 1000;

	if (y < 204)
	{
		y = 204;
	}
	grayColor->r = y; grayColor->g = y; grayColor->b = y;
}

//㷴תɫ
static void getInvertColor(RGBColorType *invertColor, RGBColorType *originalColor)
{
	UInt32		x	= ((UInt32)originalColor->r) * 299 + ((UInt32)originalColor->g) * 587 + ((UInt32)originalColor->b) * 114;
	if (((UInt32)originalColor->r) * 299 + ((UInt32)originalColor->g) * 587 + ((UInt32)originalColor->b) * 114 > 204000/*127000*/)	//
	{
		invertColor->r = 0;
		invertColor->g = 0;
		invertColor->b = 0;
	}
	else
	{
		invertColor->r = 255;
		invertColor->g = 255;
		invertColor->b = 255;
	}
}

//Ĭɫ
static void getDefaultColor(SettingType *settingP)
{
	UInt16		i;

	UIColorGetTableEntryRGB(UIFormFrame, &settingP->formFrameColor);
	UIColorGetTableEntryRGB(UIFormFill, &settingP->formFillColor);
	UIColorGetTableEntryRGB(UIObjectForeground, &settingP->textColor);
	UIColorGetTableEntryRGB(UIFormFill, &settingP->textOutlineColor);
	getInvertColor(&settingP->textOnMaskColor, &settingP->formFrameColor);
	getGrayColor(&settingP->bottomStatusFillColor, &settingP->formFrameColor);
	settingP->topStatusFillColor = settingP->formFillColor;
	settingP->focusRingColor[0].r = 0; settingP->focusRingColor[0].g = 51; settingP->focusRingColor[0].b = 204;
	settingP->focusRingColor[1].r = 0; settingP->focusRingColor[1].g = 102; settingP->focusRingColor[1].b = 255;
	settingP->focusRingColor[2].r = 51; settingP->focusRingColor[2].g = 153; settingP->focusRingColor[2].b = 255;
	settingP->focusRingColor[3].r = 102; settingP->focusRingColor[3].g = 204; settingP->focusRingColor[3].b = 255;
	settingP->focusRingColor[4].r = 153; settingP->focusRingColor[4].g = 255; settingP->focusRingColor[4].b = 255;
	settingP->focusRingColor[5].r = 204; settingP->focusRingColor[5].g = 255; settingP->focusRingColor[5].b = 255;
	for (i = hsNavFocusColorRing1; i <= hsNavFocusColorRing6; i ++)
	{
		HsNavSetFocusColor(i, &settingP->focusRingColor[i - hsNavFocusColorRing1], NULL);
	}
}